@@ -18,12 +18,13 @@ class textwindow: public abstractuicontrol {
 	
     public:
 	textwindow();
+	textwindow(const textwindow &aw);
 	textwindow(int xx1, int yy1, int xx2, int yy2, int wcolor,
 	    int noptions = 0, int tcolor = 0, const char *tfmt = 0, ...);
 	~textwindow();
 
-	void write(int x, int y, int c, string text);
-	void write(int x, int y, string text);
+	void write(int x, int y, int c, const string &text);
+	void write(int x, int y, const string &text);
 
 	void writef(int x, int y, int c, const char *fmt, ...);
 	void writef(int x, int y, const char *fmt, ...);
@@ -35,7 +36,7 @@ class textwindow: public abstractuicontrol {
 	void setcolor(int awc);
 	void setcoords(int x1, int y1, int x2, int y2);
 	void set_titlef(int color, const char *fmt, ...);
-	void set_title(int color, const string atitle);
+	void set_title(int color, const string &atitle);
 
 	void gotoxy(int x, int y);
 	void open();
