@@ -1,7 +1,7 @@
 /*
 *
 * kkconsui common routines
-* $Id: conscommon.cc,v 1.20 2002/07/06 15:16:59 konst Exp $
+* $Id: conscommon.cc,v 1.23 2002/08/14 12:03:43 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -23,13 +23,14 @@
 */
 
 #include "conscommon.h"
-#ifdef __sun__
-#include <sys/termio.h>
-#include <sys/filio.h>
-#endif
+#include <iostream>
 
 bool kintf_graph = true, kintf_refresh = true;
 
+#ifdef KTOOL_USE_FRIBIDI
+bool use_fribidi = true;
+#endif
+
 void (*kt_resize_event)(void);
 
 void kreinit(int sn) {
@@ -49,7 +50,7 @@ void kreinit(int sn) {
 void kinterface() {
 #ifdef KTOOL_USE_FRIBIDI
     // an experimental thingie
-    cout << ((char) 27) << "(K" << flush;
+    std::cout << ((char) 27) << "(K" << std::flush;
 #endif
 
     initscr();
@@ -256,6 +257,8 @@ extern "C" {
 }
 
 string makebidi(const string &buf, int lpad = 0) {
+    if(!use_fribidi) return buf;
+
     FriBidiChar *us, *out_us;
     char *outstring, *cbuf;
     int size;
