@@ -78,18 +78,11 @@ texteditor::~texteditor() {
 int texteditor::load(const string abuf, const string id) {
     int newfn = addwindow(strdup(id.c_str())), k;
     string buf = abuf;
-    string::iterator is;
     vector<string> lst;
     vector<string>::iterator i;
 
     setfnum(newfn);
 
-    is = buf.begin();
-    while((is = ::find(buf.begin(), buf.end(), '\r')) != buf.end()) {
-	buf.erase(is);
-	is++;
-    }
-
     breakintolines(buf, lst, wrap ? x2-x1 : 0);
 
     curfile->lines->empty();
@@ -266,8 +259,7 @@ int texteditor::addwindow(char *id) {
     return files->count-1;
 }
 
-void texteditor::modification(tundoaction action, const string data,
-bool connected = false, int curx = -1, int cury = -1) {
+void texteditor::modification(tundoaction action, const string &data, bool connected, int curx, int cury) {
     if(undolog && !data.empty()) {
 	undorecord *ur = new undorecord;
 	ur->x = curx < 0 ? CURCOL  : curx;
@@ -785,7 +777,7 @@ int texteditor::count_clrcodes(char *cp, int pos) {
     return k;
 }
 
-void texteditor::showline(int ln, int startx, int distance, int extrax = 0) {
+void texteditor::showline(int ln, int startx, int distance, int extrax) {
     if(!show) return;
 
     int i, n, inscount, bcolor, sxinscount, printed, j, lastoccur, q, eolstart, npos, offs;
@@ -1050,7 +1042,7 @@ bool texteditor::fix_x(bool tab) {
     return osx != curfile->sx;
 }
 
-void texteditor::eddel(bool usetabs = true) {
+void texteditor::eddel(bool usetabs) {
     char *p = CURSTRING;
     int nextlen, todelete = 1, rm;
     string deltext;
@@ -1200,7 +1192,7 @@ void texteditor::eddelline() {
     updatecursor();
 }
 
-void texteditor::edenter(bool countspaces = true) {
+void texteditor::edenter(bool countspaces) {
     char *p = CURSTRING, *r;
     string spaceins;
     
@@ -1244,7 +1236,7 @@ void texteditor::edenter(bool countspaces = true) {
     edmove(KEY_DOWN);
 }
 
-void texteditor::edmove(int k, int options = EM_TAB) {
+void texteditor::edmove(int k, int options) {
     int i, lm;
     bool fdraw = false, acted;
     bool ctrlpressed = (options & EM_CTRL) && (getctrlkeys() & CONTROL_PRESSED);
@@ -1584,6 +1576,10 @@ void texteditor::getpos(int *col, int *line) {
 int texteditor::open() {
     int k, l, go;
 
+    if((x2-x1 < 2) || (y2-y1 < 2)) {
+	return 0;
+    }
+
     if(fn < 0 || fn > files->count) setfnum(0);
 
     active = true;
@@ -1621,50 +1617,53 @@ int texteditor::open() {
 		    edmove(k, EM_TAB | EM_CTRL | EM_MANUAL);
 		    updatecursor();
 		    break;
-	    default:
-		if(prevshift) prevshift = curfile->markmode = false;
-	    switch(k) {
-		case '\r':
-		    if(insertmode || (curfile->lines->count == CURLINE+1)) {
-			edenter(smarttab);
-		    } else {
-			edmove(KEY_DOWN);
-			edmove(KEY_HOME);
-		    }
-		    break;
-		case KEY_BACKSPACE:
-		case CTRL('h'):
-		case 127:
-		    edbackspace();
-		    break;
-		case CTRL('y'):
-		    eddelline();
-		    break;
-		case KEY_TAB:
-		    inschar('\t');
-		    break;
-		case CTRL('t'):
-		    eddelword();
-		    break;
+
 		default:
-		    if(k >= 32 && k <= 255) {
-			if(!insertmode && !endofline()) eddel();
-			inschar(k);
-		    } else if(otherkeys) {
-			if(!getctrlkeys() && ((k == KEY_IC) || (k == KEY_DC)))
-			switch(k) {
-			    case KEY_IC: insertmode = !insertmode; break;
-			    case KEY_DC: eddel(); break;
+		    if(prevshift) prevshift = curfile->markmode = false;
+		switch(k) {
+		    case '\r':
+			if(insertmode || (curfile->lines->count == CURLINE+1)) {
+			    edenter(smarttab);
 			} else {
-			    l = fn;
-			    if((*otherkeys)(*this, k) == -1) return -1;
-			    if(l != fn) {
-				draw();
-				updatecursor();
+			    edmove(KEY_DOWN);
+			    edmove(KEY_HOME);
+			}
+			break;
+		    case KEY_BACKSPACE:
+		    case CTRL('h'):
+		    case 127:
+			edbackspace();
+			break;
+		    case CTRL('y'):
+			eddelline();
+			break;
+		    case KEY_TAB:
+			inschar('\t');
+			break;
+		    case CTRL('t'):
+			eddelword();
+			break;
+
+		    default:
+			if(k >= 32 && k <= 255) {
+			    if(!insertmode && !endofline()) eddel();
+			    inschar(k);
+			} else if(otherkeys) {
+			    if(!getctrlkeys() && ((k == KEY_IC) || (k == KEY_DC)))
+			    switch(k) {
+				case KEY_IC: insertmode = !insertmode; break;
+				case KEY_DC: eddel(); break;
+			    } else {
+				l = fn;
+				if((*otherkeys)(*this, k) == -1) return -1;
+				if(l != fn) {
+				    draw();
+				    updatecursor();
+				}
 			    }
 			}
-		    }
-	    }}
+		}
+	    }
 	} else {
 	    if(idle) (*idle)(*this);
 	}
@@ -1973,3 +1972,4 @@ bool hlight::operator != (const hl_kind &k) const {
 bool hlight::operator < (const hlight &ah) const {
     return kind == h_eol;
 }
+
