@@ -1,7 +1,7 @@
 /*
 *
 * kkconsui textwindow class
-* $Id: textwindow.cc,v 1.4 2001/08/06 21:32:45 konst Exp $
+* $Id: textwindow.cc,v 1.8 2002/07/03 14:38:57 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -29,7 +29,7 @@ textwindow::textwindow() {
 }
 
 textwindow::textwindow(int xx1, int yy1, int xx2, int yy2, int wcolor,
-int noptions = 0, int tcolor = 0, const char *tfmt = 0, ...) {
+int noptions, int tcolor, const char *tfmt, ...) {
     char buf[10240];
 
     setoptions(noptions);
@@ -45,6 +45,13 @@ int noptions = 0, int tcolor = 0, const char *tfmt = 0, ...) {
     }
 }
 
+textwindow::textwindow(const textwindow &aw): abstractuicontrol(aw) {
+    title = aw.title;
+    wc = aw.wc;
+    tc = aw.tc;
+    options = aw.options;
+}
+
 textwindow::~textwindow() {
 }
 
@@ -83,7 +90,7 @@ void textwindow::set_titlef(int color, const char *fmt, ...) {
     set_title(color, buf);
 }
 
-void textwindow::set_title(int color, const string atitle) {
+void textwindow::set_title(int color, const string &atitle) {
     title = atitle;
     tc = color;
     
@@ -116,19 +123,20 @@ void textwindow::writef(int x, int y, int c, const char *fmt, ...) {
     write(x, y, c, buf);
 }
 
-void textwindow::write(int x, int y, string text) {
+void textwindow::write(int x, int y, const string &text) {
     write(x, y, wc, text);
 }
 
-void textwindow::write(int x, int y, int c, string text) {
-    if(fisopen && (y < y2-y1)) {
-	text.resize(x2-x1-x);
+void textwindow::write(int x, int y, int c, const string &text) {
+    int i;
+    string dtext;
 
-	for(int k = 0; k < text.size(); k++)
-	if(text[k] > 0 && text[k] < 32) text[k] = ' ';
+    if(fisopen && (y < y2-y1)) {
+	for(i = 0; (i < text.size()) && (i < x2-x1-x); i++)
+	    dtext += KT_DISP_FILTER(text[i]);
 
 	attrset(c);
-	mvprintw(y1 + y, x1 + x, "%s", text.c_str());
+	mvprintw(y1 + y, x1 + x, "%s", dtext.c_str());
 	refresh();
     }
 }
@@ -170,7 +178,7 @@ void textwindow::redraw() {
 void textwindow::open() {
     if(!fisopen) {
 	fisopen = true;
-        screenbuffer.save(x1, y1, x2, y2);
+	screenbuffer.save(x1, y1, x2, y2);
 	redraw();
     }
 }
