@@ -37,12 +37,12 @@
 #define	FFIND_LINK	2
 #define	FFIND_DIR	4
 
-vector<string> filefind(const string mask, const string aroot, int mode = FFIND_FILE);
+vector<string> filefind(const string &mask, const string &aroot, int mode = FFIND_FILE);
 
-const string readlink(const string fname);
-const string pathfind(const string name, const string path, int amode = F_OK);
+string readlink(const string &fname);
+string pathfind(const string &name, const string &path, int amode = F_OK);
 bool mksubdirs(string dir);
-bool samefile(const string fname1, const string fname2);
+bool samefile(const string &fname1, const string &fname2);
 
 __KTOOL_BEGIN_C
 
