@@ -33,7 +33,8 @@ namespace ICQ2000 {
     enum Type {
       UserInfo,
       SMSMessage,
-      Search
+      Search,
+      ServerBasedContact
     };
 
     virtual ~RequestIDCacheValue() { }
@@ -43,12 +44,12 @@ namespace ICQ2000 {
 
   class UserInfoCacheValue : public RequestIDCacheValue {
    private:
-    Contact *m_contact;
+    ContactRef m_contact;
 
    public:
-    UserInfoCacheValue(Contact *c) : m_contact(c) { }
+    UserInfoCacheValue(ContactRef c) : m_contact(c) { }
     unsigned int getUIN() const { return m_contact->getUIN(); }
-    Contact* getContact() const { return m_contact; }
+    ContactRef getContact() const { return m_contact; }
 
     Type getType() const { return UserInfo; }
   };
@@ -59,9 +60,9 @@ namespace ICQ2000 {
 
    public:
     SMSEventCacheValue( SMSMessageEvent *ev ) : m_ev(ev) { }
-    ~SMSEventCacheValue() { delete m_ev; }
+    virtual ~SMSEventCacheValue() { delete m_ev; }
     SMSMessageEvent* getEvent() const { return m_ev; }
-    Contact* getContact() const { return m_ev->getContact(); }
+    ContactRef getContact() const { return m_ev->getContact(); }
 
     Type getType() const { return SMSMessage; }
   };
@@ -76,9 +77,23 @@ namespace ICQ2000 {
     Type getType() const { return Search; }
   };
 
+  class ServerBasedContactCacheValue : public RequestIDCacheValue {
+   private:
+    ServerBasedContactEvent *m_ev;
+
+   public:
+    ServerBasedContactCacheValue( ServerBasedContactEvent *ev ) : m_ev(ev) { }
+    ServerBasedContactEvent *getEvent() const { return m_ev; }
+    Type getType() const { return ServerBasedContact; }
+  };
+
   class RequestIDCache : public Cache<unsigned int, RequestIDCacheValue*> {
    public:
     RequestIDCache() { }
+    ~RequestIDCache()
+    {
+      removeAll();
+    }
 
     SigC::Signal1<void,RequestIDCacheValue*> expired;
     
@@ -92,24 +107,6 @@ namespace ICQ2000 {
       Cache<unsigned int, RequestIDCacheValue*>::removeItem(l);
     }
 
-    void removeContact(Contact *c) {
-      literator curr = m_list.begin();
-      literator next = curr;
-      while ( curr != m_list.end() ) {
-	++next;
-	RequestIDCacheValue *cv = (*curr).getValue();
-	if ( cv->getType() == RequestIDCacheValue::UserInfo ) {
-	  UserInfoCacheValue *ccv = static_cast<UserInfoCacheValue*>(cv);
-	  if (ccv->getContact() == c) removeItem(curr);
-	} else if (cv->getType() == RequestIDCacheValue::SMSMessage ) {
-	  SMSEventCacheValue *ccv = static_cast<SMSEventCacheValue*>(cv);
-	  if (ccv->getContact() == c) removeItem(curr);
-	}
-	
-	curr = next;
-      }
-    }
-
   };
   
 }
