@@ -29,8 +29,12 @@
 
 #include <sigc++/bind.h>
 
+#include <vector>
+
+using std::string;
 using std::ostringstream;
 using std::endl;
+using std::vector;
 
 namespace ICQ2000 {
 
@@ -100,6 +104,9 @@ namespace ICQ2000 {
     m_lower_port = 0;
     m_upper_port = 0;
 
+    m_default_group = 0x13a0;
+    m_default_group_name = "General";
+
     m_cookiecache.setDefaultTimeout(30);
     // 30 seconds is hopefully enough for even the slowest connections
     m_cookiecache.expired.connect( slot(this,&Client::ICBMCookieCache_expired_cb) );
@@ -459,6 +466,7 @@ namespace ICQ2000 {
 	  c->setEmail( snac->getEmail() );
 	  c->setFirstName( snac->getFirstName() );
 	  c->setLastName( snac->getLastName() );
+	  c->setAuthReq(snac->getAuthReq());
 	}
       }
       
@@ -575,6 +583,31 @@ namespace ICQ2000 {
 	SignalLog(LogEvent::WARN, e.what());
       }
 
+    } else if (snac->getType() == SrvResponseSNAC::RandomChatFound) {
+
+      if ( m_reqidcache.exists( snac->RequestID() ) ) {
+	RequestIDCacheValue *v = m_reqidcache[ snac->RequestID() ];
+
+	if ( v->getType() == RequestIDCacheValue::Search ) {
+	  SearchCacheValue *sv = static_cast<SearchCacheValue*>(v);
+
+	  SearchResultEvent *ev = sv->getEvent();
+
+	  ContactRef c = new Contact( snac->getUIN() );
+	  ContactList& cl = ev->getContactList();
+	  ev->setLastContactAdded( cl.add(c) );
+	  ev->setFinished(true);
+
+	  search_result.emit(ev);
+
+	  delete ev;
+	  m_reqidcache.remove( snac->RequestID() );
+	  
+	} else {
+	  SignalLog(LogEvent::WARN, "Request ID cached value is not for a Search request");
+	}
+
+      }
     }
   }
   
@@ -808,7 +841,7 @@ namespace ICQ2000 {
     SignalLog(LogEvent::INFO, "Sending Rate Info Request");
     FLAPwrapSNACandSend( RequestRateInfoSNAC() );
   }
-  
+
   void Client::SendRateInfoAck() {
     SignalLog(LogEvent::INFO, "Sending Rate Info Ack");
     FLAPwrapSNACandSend( RateInfoAckSNAC() );
@@ -831,9 +864,9 @@ namespace ICQ2000 {
     // know the listening port and ip
     if (m_in_dc) {
       if (m_use_portrange) {
-        m_listenServer.StartServer(m_lower_port, m_upper_port);
+	m_listenServer.StartServer(m_lower_port, m_upper_port);
       } else {
-        m_listenServer.StartServer();
+	m_listenServer.StartServer();
       }
       SignalAddSocket( m_listenServer.getSocketHandle(), SocketEvent::READ );
       ostringstream ostr;
@@ -848,7 +881,7 @@ namespace ICQ2000 {
 
     if (m_invisible_wanted)
       FLAPwrapSNAC(b, AddVisibleSNAC(m_visible_list) );
-        
+	
     SetStatusSNAC sss(Contact::MapStatusToICQStatus(m_status_wanted, m_invisible_wanted), m_web_aware);
 
     sss.setSendExtra(true);
@@ -986,7 +1019,7 @@ namespace ICQ2000 {
 	ParseCh4(sb,seq_num);
 	break;
       default:
-	ostr << "FLAP on unrecognised channel 0x" << hex << (int)channel;
+	ostr << "FLAP on unrecognised channel 0x" << std::hex << (int)channel;
 	SignalLog(LogEvent::WARN, ostr.str());
 	break;
       }
@@ -997,7 +1030,7 @@ namespace ICQ2000 {
 	 * as it probably means they are faulty
 	 */
 	ostringstream ostr;
-	ostr  << "Buffer pointer not at end after parsing FLAP was: 0x" << hex << sb.pos()
+	ostr  << "Buffer pointer not at end after parsing FLAP was: 0x" << std::hex << sb.pos()
 	      << " should be: 0x" << sb.size();
 	SignalLog(LogEvent::WARN, ostr.str());
       }
@@ -1139,18 +1172,86 @@ namespace ICQ2000 {
       switch(snac->Subtype()) {
       case SNAC_SBL_List_From_Server:
 	SignalLog(LogEvent::INFO, "Received server-based list from server\n");
-        SBLListSNAC *sbs = static_cast<SBLListSNAC*>(snac);
-        SignalServerBasedContactList( sbs->getContactList() );
+	SignalServerBasedContactList( static_cast<SBLListSNAC*>(snac)->getContactList() );
+	break;
+      case SNAC_SBL_Edit_Access_Granted:
+	SignalLog(LogEvent::INFO, "Access to the server-based contact list was granted\n");
+	break;
+      case SNAC_SBL_Modification_Ack: {
+	vector<ModificationAckSBLSNAC::Result> r = static_cast<ModificationAckSBLSNAC*>(snac)->getResults();
+	vector<ModificationAckSBLSNAC::Result>::iterator ir;
+	vector<ServerBasedContactEvent::UploadResult> updresults;
+	vector<ServerBasedContactEvent::UploadResult>::iterator iur;
+
+	for(ir = r.begin(); ir != r.end(); ++ir)
+	switch( *ir ) {
+	  case ModificationAckSBLSNAC::Success:
+	    SignalLog(LogEvent::INFO, "Server-based contact list modification succeeded\n");
+	    updresults.push_back(ServerBasedContactEvent::Success);
+	    break;
+	  case ModificationAckSBLSNAC::Failed:
+	    SignalLog(LogEvent::INFO, "Server-based contact list modification failed\n");
+	    updresults.push_back(ServerBasedContactEvent::Failed);
+	    break;
+	  case ModificationAckSBLSNAC::AuthRequired:
+	    SignalLog(LogEvent::INFO, "Authentification is required to perform the server-based modification\n");
+	    updresults.push_back(ServerBasedContactEvent::AuthRequired);
+	    break;
+	  case ModificationAckSBLSNAC::AlreadyExists:
+	    SignalLog(LogEvent::INFO, "Already exists on the server-based contact list\n");
+	    break;
+	  }
+
+	if(!updresults.empty()) {
+	  if( m_reqidcache.exists( snac->RequestID() ) ) {
+	    RequestIDCacheValue *v = m_reqidcache[ snac->RequestID() ];
+
+	    if ( v->getType() == RequestIDCacheValue::ServerBasedContact ) {
+	      ServerBasedContactCacheValue *sv = static_cast<ServerBasedContactCacheValue*>(v);
+	      ServerBasedContactEvent *ev = sv->getEvent();
+
+	      ContactList &cl = ev->getContactList();
+	      ContactList::iterator ic = cl.begin();
+
+	      iur = updresults.begin();
+
+	      while(iur != updresults.end() && ic != cl.end()) {
+		if(*iur == ServerBasedContactEvent::Success) {
+		    (*ic)->setServerBased(ev->getType() == ServerBasedContactEvent::Upload);
+
+		    ContactRef ct = getContact((*ic)->getUIN());
+		    if(ct.get()) {
+		      ct->setServerBased(ev->getType() == ServerBasedContactEvent::Upload);
+		    }
+		}
+
+		++iur;
+		++ic;
+	      }
+
+	      ev->setUploadResults(updresults);
+	      server_based_contact_list.emit(ev);
+
+	      delete ev;
+	      m_reqidcache.remove( snac->RequestID() );
+	    }
+
+	  } else {
+	    SignalLog(LogEvent::WARN, "Request ID cached value is not for a server-base contacts upload request");
+	  }
+
+	}
+
+	}
 	break;
       }
       break;
-	
-	
+
     } // switch(Family)
 
     if (dynamic_cast<RawSNAC*>(snac)) {
       ostringstream ostr;
-      ostr << "Unknown SNAC packet received - Family: 0x" << hex << snac->Family()
+      ostr << "Unknown SNAC packet received - Family: 0x" << std::hex << snac->Family()
 	   << " Subtype: 0x" << snac->Subtype();
       SignalLog(LogEvent::WARN, ostr.str());
     }
@@ -1174,11 +1275,11 @@ namespace ICQ2000 {
 	RedirectTLV *r = static_cast<RedirectTLV*>(tlvlist[TLV_Redirect]);
 	ostringstream ostr;
 	ostr << "Redirected to: " << r->getHost();
-	if (r->getPort() != 0) ostr << " port: " << dec << r->getPort();
+	if (r->getPort() != 0) ostr << " port: " << std::dec << r->getPort();
 	SignalLog(LogEvent::INFO, ostr.str());
 
 	m_bosHostname = r->getHost();
-	if (!m_bosOverridePort)	{
+	if (!m_bosOverridePort) {
 	  if (r->getPort() != 0) m_bosPort = r->getPort();
 	  else m_bosPort = m_authorizerPort;
 	}
@@ -1660,8 +1761,13 @@ namespace ICQ2000 {
     ev->setDirect(false);
     ICQMessageEvent *cev;
     if ((cev = dynamic_cast<ICQMessageEvent*>(ev)) != NULL) cev->setOfflineMessage(true);
-    
+
+    if (ev->getType() == MessageEvent::AuthReq) {
+      ev->getContact()->setAuthAwait(true);
+    }
+
     messageack.emit(ev);
+    delete ist;
   }
   
   void Client::PingServer() {
@@ -1671,7 +1777,6 @@ namespace ICQ2000 {
     Send(b);
   }
 
-
   void Client::uploadSelfDetails()
   {
     Buffer b(&m_translator);
@@ -1683,7 +1788,56 @@ namespace ICQ2000 {
     
     Send(b);
   }
-    
+
+  void Client::uploadServerBasedContactList()
+  {
+    uploadServerBasedContactList(m_contact_list);
+  }
+
+  void Client::uploadServerBasedContactList(const ContactList &l)
+  {
+    Buffer b(&m_translator);
+
+    FLAPwrapSNAC( b, EditReqAccessSBLSNAC() );
+    FLAPwrapSNAC( b, AddItemSBLSNAC(m_default_group_name, m_default_group) );
+
+    ServerBasedContactEvent *ev = new ServerBasedContactEvent(ServerBasedContactEvent::Upload, l);
+    unsigned int reqid = NextRequestID();
+    m_reqidcache.insert( reqid, new ServerBasedContactCacheValue( ev ) );
+
+    AddItemSBLSNAC ssnac(l);
+    ssnac.setRequestID( reqid );
+    FLAPwrapSNAC( b, ssnac );
+
+    FLAPwrapSNAC( b, EditFinishSBLSNAC() );
+
+    Send(b);
+  }
+
+  void Client::removeServerBasedContactList()
+  {
+    removeServerBasedContactList(m_contact_list);
+  }
+
+  void Client::removeServerBasedContactList(const ContactList &l)
+  {
+    Buffer b(&m_translator);
+
+    FLAPwrapSNAC( b, EditReqAccessSBLSNAC() );
+
+    ServerBasedContactEvent *ev = new ServerBasedContactEvent(ServerBasedContactEvent::Remove, l);
+    unsigned int reqid = NextRequestID();
+    m_reqidcache.insert( reqid, new ServerBasedContactCacheValue( ev ) );
+
+    RemoveItemSBLSNAC ssnac(l);
+    ssnac.setRequestID( reqid );
+    FLAPwrapSNAC( b, ssnac );
+
+    FLAPwrapSNAC( b, EditFinishSBLSNAC() );
+
+    Send(b);
+  }
+
   /**
    *  Set your status. This is used to set your status, as well as to
    *  connect and disconnect from the network. When you wish to
@@ -1742,7 +1896,7 @@ namespace ICQ2000 {
       // start connecting if not already
       if (m_state == NOT_CONNECTED) {
 	ConnectingEvent ev;
-        connecting.emit(&ev);
+	connecting.emit(&ev);
 	ConnectAuthorizer(AUTH_AWAITING_CONN_ACK);
       }
     }
@@ -1780,6 +1934,23 @@ namespace ICQ2000 {
     }
   }
 
+  void Client::setRandomChatGroup(unsigned short group) {
+    if(m_random_group != group && m_state != NOT_CONNECTED) {
+      m_random_group = group;
+
+      unsigned int reqid = NextRequestID();
+      SrvSetRandomChatGroup ssnac( m_self->getUIN(), group );
+
+      SignalLog(LogEvent::INFO, "Setting random chat group");
+      FLAPwrapSNACandSend( ssnac );
+    }
+  }
+
+  void Client::setServerSideGroup(const std::string &group_name, unsigned short group_id) {
+    m_default_group = group_id;
+    m_default_group_name = group_name;
+  }
+
   /**
    *  Get your current status.
    *
@@ -1902,6 +2073,10 @@ namespace ICQ2000 {
 
     if (!m_contact_list.exists(c->getUIN())) {
       m_contact_list.add(c);
+
+      if(!c->getServerBased())
+	c->setServerSideInfo(m_default_group, 0);
+
       c->status_change_signal.connect( contact_status_change_signal.slot() );
       c->userinfo_change_signal.connect( contact_userinfo_change_signal.slot() );
     }
@@ -1966,9 +2141,22 @@ namespace ICQ2000 {
       m_invisible_list.remove(uin);
     }
   }
-  
+
   void Client::SignalServerBasedContactList(const ContactList& l) {
-    ServerBasedContactEvent ev(l);
+    ContactRef ct;
+    ContactList::const_iterator curr = l.begin();
+
+    while(curr != l.end()) {
+      ct = getContact((*curr)->getUIN());
+      if(ct.get()) {
+        ct->setServerBased(true);
+	ct->setServerSideInfo((*curr)->getServerSideGroupID(), (*curr)->getServerSideID());
+      }
+
+      ++curr;
+    }
+
+    ServerBasedContactEvent ev(ServerBasedContactEvent::Fetch, l);
     server_based_contact_list.emit(&ev);
   }
 
@@ -2143,7 +2331,7 @@ namespace ICQ2000 {
     return ev;
   }
 
-  SearchResultEvent* Client::searchForContacts(const std::string& keyword)
+  SearchResultEvent* Client::searchForContacts(const string& keyword)
   {
     SearchResultEvent *ev = new SearchResultEvent( SearchResultEvent::Keyword );
     unsigned int reqid = NextRequestID();
@@ -2158,6 +2346,21 @@ namespace ICQ2000 {
     return ev;
   }
 
+  SearchResultEvent* Client::searchForContacts(RandomChatGroup group)
+  {
+    SearchResultEvent *ev = new SearchResultEvent( SearchResultEvent::RandomChat );
+    unsigned int reqid = NextRequestID();
+    m_reqidcache.insert( reqid, new SearchCacheValue( ev ) );
+    
+    SrvRequestRandomChat ssnac( m_self->getUIN(), group );
+    ssnac.setRequestID( reqid );
+    
+    SignalLog(LogEvent::INFO, "Sending contact random chat search request");
+    FLAPwrapSNACandSend( ssnac );
+    
+    return ev;
+  }
+
   void Client::Disconnect(DisconnectedEvent::Reason r) {
     if (m_state == NOT_CONNECTED) return;
 
