@@ -29,6 +29,7 @@
 
 #include <libicq2000/time_extra.h>
 
+using std::string;
 using std::auto_ptr;
 using std::istringstream;
 
@@ -148,9 +149,9 @@ namespace ICQ2000 {
     b << (unsigned short)0x000e;
     b << m_my_uin;
 
-    b << (unsigned short)2000	/* type 9808 */
+    b << (unsigned short)2000   /* type 9808 */
       << (unsigned short)m_requestID /* low word of the request ID */
-      << (unsigned short)1311	/* subtype (unsigned short)1311 */
+      << (unsigned short)1311   /* subtype (unsigned short)1311 */
       << m_user_uin;
     
   }
@@ -174,9 +175,9 @@ namespace ICQ2000 {
 
     b << m_my_uin;
 
-    b << (unsigned short)2000	/* type 9808 */
+    b << (unsigned short)2000   /* type 9808 */
       << (unsigned short)m_requestID /* low word of the request ID */
-      << (unsigned short)0x0515;	/* subtype wp-short-request */
+      << (unsigned short)0x0515;        /* subtype wp-short-request */
     b.PackUint16TranslatedNull(m_firstname);
     b.PackUint16TranslatedNull(m_lastname);
     b.PackUint16TranslatedNull(m_nickname);
@@ -213,34 +214,34 @@ namespace ICQ2000 {
 
     b << m_my_uin;
 
-    b << (unsigned short)2000	/* type 9808 */
+    b << (unsigned short)2000   /* type 9808 */
       << (unsigned short)m_requestID /* low word of the request ID */
-      << (unsigned short)0x0533;	/* subtype wp-full-request */
+      << (unsigned short)0x0533;        /* subtype wp-full-request */
     b.PackUint16TranslatedNull(m_firstname);
     b.PackUint16TranslatedNull(m_lastname);
     b.PackUint16TranslatedNull(m_nickname);
     b.PackUint16TranslatedNull(m_email);
-    b << (unsigned short)m_min_age;		// minimum age
-    b << (unsigned short)m_max_age;		// maximum age
-    b << (unsigned char)m_sex;			// sex
+    b << (unsigned short)m_min_age;             // minimum age
+    b << (unsigned short)m_max_age;             // maximum age
+    b << (unsigned char)m_sex;                  // sex
     b << (unsigned char)m_language;             // language
     b.PackUint16TranslatedNull(m_city);             // city
     b.PackUint16TranslatedNull(m_state);            // state
     b << (unsigned short)m_country;             // country
-    b.PackUint16TranslatedNull(m_company_name);	// company name
-    b.PackUint16TranslatedNull(m_department);	// department
-    b.PackUint16TranslatedNull(m_position);		// position
-    b << (unsigned char)0x00;			// occupation
-    b << (unsigned short)0x0000;		// past info category
-    b.PackUint16TranslatedNull("");			//           description
-    b << (unsigned short)0x0000;		// interests category
-    b.PackUint16TranslatedNull("");			//           description
-    b << (unsigned short)0x0000;		// affiliation/organization
-    b.PackUint16TranslatedNull("");			//           description
-    b << (unsigned short)0x0000;		// homepage category
-    b.PackUint16TranslatedNull("");			//           description
+    b.PackUint16TranslatedNull(m_company_name); // company name
+    b.PackUint16TranslatedNull(m_department);   // department
+    b.PackUint16TranslatedNull(m_position);             // position
+    b << (unsigned char)0x00;                   // occupation
+    b << (unsigned short)0x0000;                // past info category
+    b.PackUint16TranslatedNull("");                     //           description
+    b << (unsigned short)0x0000;                // interests category
+    b.PackUint16TranslatedNull("");                     //           description
+    b << (unsigned short)0x0000;                // affiliation/organization
+    b.PackUint16TranslatedNull("");                     //           description
+    b << (unsigned short)0x0000;                // homepage category
+    b.PackUint16TranslatedNull("");                     //           description
     b << (unsigned char)(m_only_online ? 0x01 : 0x00);
-                                                // only-online flag
+						// only-online flag
     b.setAutoSizeMarker(m1);
     b.setAutoSizeMarker(m2);
   }
@@ -260,7 +261,7 @@ namespace ICQ2000 {
 
     b << m_my_uin;
 
-    b << (unsigned short)2000	     /* type 9808 */
+    b << (unsigned short)2000        /* type 9808 */
       << (unsigned short)m_requestID /* low word of the request ID */
       << (unsigned short)0x055F      /* subtype keyword-search */
       << (unsigned short)0x0226;     /* unknown */
@@ -295,118 +296,174 @@ namespace ICQ2000 {
     : m_my_uin(my_uin), m_main_home_info(main_home_info) { }
     
   void SrvUpdateMainHomeInfo::OutputBody(Buffer& b) const {
-        b << (unsigned short)0x0001;
+	b << (unsigned short)0x0001;
 
-        Buffer::marker m1 = b.getAutoSizeShortMarker();
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
     
-        b.setLittleEndian();
-        Buffer::marker m2 = b.getAutoSizeShortMarker();
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
 
-        b << m_my_uin;
+	b << m_my_uin;
 
-        b << (unsigned short)2000   /* type 9808 */
-            << (unsigned short)m_requestID /* low word of the request ID */
-            << (unsigned short)0x03ea; /* subtype */
-        b.PackUint16TranslatedNull(m_main_home_info.alias);     // alias
-        b.PackUint16TranslatedNull(m_main_home_info.firstname); // first name
-        b.PackUint16TranslatedNull(m_main_home_info.lastname);  // last name
-        b.PackUint16TranslatedNull(m_main_home_info.email);	    // email
-        b.PackUint16TranslatedNull(m_main_home_info.city);	    // city
-        b.PackUint16TranslatedNull(m_main_home_info.state);     // state
-        b.PackUint16TranslatedNull(m_main_home_info.phone);     // phone
-        b.PackUint16TranslatedNull(m_main_home_info.fax);       // fax
-        b.PackUint16TranslatedNull(m_main_home_info.street);    // street
-        b.PackUint16TranslatedNull(m_main_home_info.getMobileNo());  // cellular
-        b.PackUint16TranslatedNull(m_main_home_info.zip);       // zip
-        b << m_main_home_info.country;
-        b << m_main_home_info.timezone;
-        unsigned char publish_email = 0;
-        b << publish_email;
-        b.setAutoSizeMarker(m1);
-        b.setAutoSizeMarker(m2);
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x03ea; /* subtype */
+	b.PackUint16TranslatedNull(m_main_home_info.alias);     // alias
+	b.PackUint16TranslatedNull(m_main_home_info.firstname); // first name
+	b.PackUint16TranslatedNull(m_main_home_info.lastname);  // last name
+	b.PackUint16TranslatedNull(m_main_home_info.email);         // email
+	b.PackUint16TranslatedNull(m_main_home_info.city);          // city
+	b.PackUint16TranslatedNull(m_main_home_info.state);     // state
+	b.PackUint16TranslatedNull(m_main_home_info.phone);     // phone
+	b.PackUint16TranslatedNull(m_main_home_info.fax);       // fax
+	b.PackUint16TranslatedNull(m_main_home_info.street);    // street
+	b.PackUint16TranslatedNull(m_main_home_info.getMobileNo());  // cellular
+	b.PackUint16TranslatedNull(m_main_home_info.zip);       // zip
+	b << m_main_home_info.country;
+	b << m_main_home_info.timezone;
+	unsigned char publish_email = 0;
+	b << publish_email;
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
     }
     
   SrvUpdateWorkInfo::SrvUpdateWorkInfo(unsigned int my_uin, const Contact::WorkInfo& work_info)
     : m_my_uin(my_uin), m_work_info(work_info) { }
     
   void SrvUpdateWorkInfo::OutputBody(Buffer& b) const {
-        b << (unsigned short)0x0001;
+	b << (unsigned short)0x0001;
 
-        Buffer::marker m1 = b.getAutoSizeShortMarker();
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
     
-        b.setLittleEndian();
-        Buffer::marker m2 = b.getAutoSizeShortMarker();
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
 
-        b << m_my_uin;
+	b << m_my_uin;
 
-        b << (unsigned short)2000   /* type 9808 */
-            << (unsigned short)m_requestID /* low word of the request ID */
-            << (unsigned short)0x03f3; /* subtype */
-        b.PackUint16TranslatedNull(m_work_info.city);     	    // city
-        b.PackUint16TranslatedNull(m_work_info.state);     	    // state
-        b << (unsigned short)0x0000
-          << (unsigned short)0x0000;
-        b.PackUint16TranslatedNull(m_work_info.street);         // street
-        b.PackUint16TranslatedNull(m_work_info.zip);     	    // zip
-        b << m_work_info.country;			    // country-code
-        b.PackUint16TranslatedNull(m_work_info.company_name);   // company: name
-        b.PackUint16TranslatedNull(m_work_info.company_dept);   // company: department
-        b.PackUint16TranslatedNull(m_work_info.company_position); // company: position
-        b << (unsigned short)0x0000;
-        b.PackUint16TranslatedNull(m_work_info.company_web);    // company: homepage
-        b.setAutoSizeMarker(m1);
-        b.setAutoSizeMarker(m2);
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x03f3; /* subtype */
+	b.PackUint16TranslatedNull(m_work_info.city);               // city
+	b.PackUint16TranslatedNull(m_work_info.state);              // state
+	b << (unsigned short)0x0000
+	  << (unsigned short)0x0000;
+	b.PackUint16TranslatedNull(m_work_info.street);         // street
+	b.PackUint16TranslatedNull(m_work_info.zip);                // zip
+	b << m_work_info.country;                           // country-code
+	b.PackUint16TranslatedNull(m_work_info.company_name);   // company: name
+	b.PackUint16TranslatedNull(m_work_info.company_dept);   // company: department
+	b.PackUint16TranslatedNull(m_work_info.company_position); // company: position
+	b << (unsigned short)0x0000;
+	b.PackUint16TranslatedNull(m_work_info.company_web);    // company: homepage
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
     }
     
   SrvUpdateHomepageInfo::SrvUpdateHomepageInfo(unsigned int my_uin, const Contact::HomepageInfo& homepage_info)
     : m_my_uin(my_uin), m_homepage_info(homepage_info) { }
     
   void SrvUpdateHomepageInfo::OutputBody(Buffer& b) const {
-        b << (unsigned short)0x0001;
+	b << (unsigned short)0x0001;
 
-        Buffer::marker m1 = b.getAutoSizeShortMarker();
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
     
-        b.setLittleEndian();
-        Buffer::marker m2 = b.getAutoSizeShortMarker();
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
 
-        b << m_my_uin;
+	b << m_my_uin;
 
-        b << (unsigned short)2000   /* type 9808 */
-            << (unsigned short)m_requestID /* low word of the request ID */
-            << (unsigned short)0x03fd; /* subtype */
-        b << (unsigned char)m_homepage_info.age;	    // age
-        b << (unsigned char)0x00;
-        b << (unsigned char)m_homepage_info.sex;	    // sex-code
-        b.PackUint16TranslatedNull(m_homepage_info.homepage);
-        b << (unsigned short)m_homepage_info.birth_year;
-        b << (unsigned char)m_homepage_info.birth_month;
-        b << (unsigned char)m_homepage_info.birth_day;
-        b << (unsigned char)m_homepage_info.lang1;
-        b << (unsigned char)m_homepage_info.lang2;
-        b << (unsigned char)m_homepage_info.lang3;
-        b.setAutoSizeMarker(m1);
-        b.setAutoSizeMarker(m2);
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x03fd; /* subtype */
+	b << (unsigned char)m_homepage_info.age;            // age
+	b << (unsigned char)0x00;
+	b << (unsigned char)m_homepage_info.sex;            // sex-code
+	b.PackUint16TranslatedNull(m_homepage_info.homepage);
+	b << (unsigned short)m_homepage_info.birth_year;
+	b << (unsigned char)m_homepage_info.birth_month;
+	b << (unsigned char)m_homepage_info.birth_day;
+	b << (unsigned char)m_homepage_info.lang1;
+	b << (unsigned char)m_homepage_info.lang2;
+	b << (unsigned char)m_homepage_info.lang3;
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
     }
     
   SrvUpdateAboutInfo::SrvUpdateAboutInfo(unsigned int my_uin, const string& about_info)
     : m_my_uin(my_uin), m_about_info(about_info) { }
     
   void SrvUpdateAboutInfo::OutputBody(Buffer& b) const {
-        b << (unsigned short)0x0001;
+	b << (unsigned short)0x0001;
 
-        Buffer::marker m1 = b.getAutoSizeShortMarker();
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
     
-        b.setLittleEndian();
-        Buffer::marker m2 = b.getAutoSizeShortMarker();
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
 
-        b << m_my_uin;
+	b << m_my_uin;
 
-        b << (unsigned short)2000   /* type 9808 */
-            << (unsigned short)m_requestID /* low word of the request ID */
-            << (unsigned short)0x0406; /* subtype */
-        b.PackUint16TranslatedNull(m_about_info);
-        b.setAutoSizeMarker(m1);
-        b.setAutoSizeMarker(m2);
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x0406; /* subtype */
+	b.PackUint16TranslatedNull(m_about_info);
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
+    }
+    
+  SrvRequestRandomChat::SrvRequestRandomChat(unsigned int my_uin, unsigned short random_group)
+    : m_my_uin(my_uin), m_random_group(random_group) { }
+    
+  void SrvRequestRandomChat::OutputBody(Buffer& b) const {
+	b << (unsigned short)0x0001;
+
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
+    
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
+
+	b << m_my_uin;
+
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x074E; /* subtype */
+
+	b << (unsigned short)m_random_group;
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
+    }
+
+  SrvSetRandomChatGroup::SrvSetRandomChatGroup(unsigned int my_uin, unsigned short random_group)
+    : m_my_uin(my_uin), m_random_group(random_group) { }
+    
+  void SrvSetRandomChatGroup::OutputBody(Buffer& b) const {
+	int i;
+	b << (unsigned short)0x0001;
+
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
+    
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
+
+	b << m_my_uin;
+
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x0758; /* subtype */
+
+	b << (unsigned short)m_random_group;
+
+	b << (unsigned short) 0x0000;
+	b << (unsigned short) 0x0220;
+	for(i = 0; i < 6; i++) b << (unsigned short) 0x0000;
+	b << (char) 0x0;
+	for(i = 0; i < 3; i++) b << (unsigned short) 0x0000;
+	b << (char) 0x50;
+	for(i = 0; i < 3; i++) b << (char) 0x0;
+	b << (char) 0x03;
+	for(i = 0; i < 3; i++) b << (char) 0x0;
+
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
     }
     
   SrvResponseSNAC::SrvResponseSNAC() : m_icqsubtype(NULL) { }
@@ -529,6 +586,12 @@ namespace ICQ2000 {
     case SrvResponse_AckAboutInfoChange:
       ParseInfoChangeAck(b, subtype);
       break;
+    case SrvResponse_RandomChatFound:
+      ParseRandomChatFound(b);
+      break;
+    case SrvResponse_AckRandomGroupSet:
+      ParseRandomGroupSetAck(b);
+      break;
     default:
       throw ParseException("Unknown ICQ subtype for Server response SNAC");
     }
@@ -677,8 +740,8 @@ namespace ICQ2000 {
       b.UnpackUint16TranslatedNull(m_main_home_info.alias);     // alias
       b.UnpackUint16TranslatedNull(m_main_home_info.firstname); // first name
       b.UnpackUint16TranslatedNull(m_main_home_info.lastname);  // last name
-      b.UnpackUint16TranslatedNull(m_main_home_info.email);	    // email
-      b.UnpackUint16TranslatedNull(m_main_home_info.city);	    // city
+      b.UnpackUint16TranslatedNull(m_main_home_info.email);         // email
+      b.UnpackUint16TranslatedNull(m_main_home_info.city);          // city
       b.UnpackUint16TranslatedNull(m_main_home_info.state);     // state
       b.UnpackUint16TranslatedNull(m_main_home_info.phone);     // phone
       b.UnpackUint16TranslatedNull(m_main_home_info.fax);       // fax
@@ -730,7 +793,7 @@ namespace ICQ2000 {
     case SrvResponse_WorkInfo: {
       b.UnpackUint16TranslatedNull(m_work_info.city);
       b.UnpackUint16TranslatedNull(m_work_info.state);
-      string s;	// these fields are incorrect in the spec
+      string s; // these fields are incorrect in the spec
       b.UnpackUint16TranslatedNull(s);
       b.UnpackUint16TranslatedNull(s);
       b.UnpackUint16TranslatedNull(m_work_info.street);
@@ -859,4 +922,19 @@ namespace ICQ2000 {
 
   }
 
+  void SrvResponseSNAC::ParseRandomChatFound(Buffer& b) {
+    unsigned char wb;
+
+    m_empty_contact = false;
+    m_type = RandomChatFound;
+
+    b >> wb;
+    b >> m_uin;
+    b >> wb;
+  }
+
+  void SrvResponseSNAC::ParseRandomGroupSetAck(Buffer& b) {
+    m_type = RandomGroupSet;
+  }
+
 }
