@@ -23,6 +23,8 @@
 
 #include <libicq2000/Contact.h>
 
+using std::string;
+
 namespace ICQ2000 {
 
   // ============================================================================
@@ -244,8 +246,24 @@ namespace ICQ2000 {
   //  ServerBasedContactEvent
   // ============================================================================
 
-  ServerBasedContactEvent::ServerBasedContactEvent(const ContactList& l) : m_clist(l) { }
+  ServerBasedContactEvent::ServerBasedContactEvent(SBLType t, const ContactList& l) : m_type(t), m_clist(l) { }
   ContactList& ServerBasedContactEvent::getContactList() { return m_clist; }
+  void ServerBasedContactEvent::setUploadResults(const std::vector<ServerBasedContactEvent::UploadResult> &v) { m_results = v; }
+
+  std::map<unsigned int, ServerBasedContactEvent::UploadResult> ServerBasedContactEvent::getUploadResults() const {
+    std::map<unsigned int, ServerBasedContactEvent::UploadResult> r;
+
+    ContactList::const_iterator curr = m_clist.begin();
+    std::vector<ServerBasedContactEvent::UploadResult>::const_iterator ir = m_results.begin();
+
+    while(curr != m_clist.end() && ir != m_results.end()) {
+      r[(*curr)->getUIN()] = *ir;
+      ++curr;
+      ++ir;
+    }
+
+    return r;
+  }
 
   // ============================================================================
   //  Contact Event
@@ -928,7 +946,7 @@ namespace ICQ2000 {
    * @param t time the message was sent
    */
   AuthAckEvent::AuthAckEvent(ContactRef c, const string& msg,
-                             bool granted, time_t t)
+			     bool granted, time_t t)
     : ICQMessageEvent(c), m_message(msg), m_granted(granted) {
     setOfflineMessage(true);
     m_time = t;
@@ -1066,14 +1084,14 @@ namespace ICQ2000 {
    * @param maxavg the maximum average
    */
   RateInfoChangeEvent::RateInfoChangeEvent(unsigned short code, 
-                                           unsigned short rateclass,
-                                           unsigned int windowsize,
-                                           unsigned int clear,
-                                           unsigned int alert,
-                                           unsigned int limit,
-                                           unsigned int disconnect,
-                                           unsigned int currentavg,
-                                           unsigned int maxavg) 
+					   unsigned short rateclass,
+					   unsigned int windowsize,
+					   unsigned int clear,
+					   unsigned int alert,
+					   unsigned int limit,
+					   unsigned int disconnect,
+					   unsigned int currentavg,
+					   unsigned int maxavg) 
     : m_code(code), m_rateclass(rateclass), m_windowsize(windowsize),
       m_clear(clear), m_alert(alert), m_limit(limit), m_disconnect(disconnect), 
       m_currentavg(currentavg), m_maxavg(maxavg) { }
