@@ -95,6 +95,9 @@ void accountmanager::exec() {
 		    if(pname == irc)
 			t.addnode(n, 0, citem+11, _(" Channel manager "));
 
+		    if(capab & hoptCanSyncList)
+			t.addnode(n, 0, citem+12, _(" Synchronize contact list "));
+
 		    t.addnode(n, 0, citem+8, _(" Drop "));
 		}
 	    }
@@ -114,16 +117,20 @@ void accountmanager::exec() {
 	    switch(action) {
 		case 1:
 		    tmp = face.inputstr(spname + _(" user name: "), account.nickname);
-		    if(!tmp.empty()) account.nickname = tmp;
+		    if(face.getlastinputkey() != KEY_ESC && !tmp.empty()) account.nickname = tmp;
 		    break;
+
 		case 2:
 		    i = strtoul(face.inputstr(spname + _(" uin: "), i2str(account.uin)).c_str(), 0, 0);
-		    if(i) account.uin = i;
+		    if(face.getlastinputkey() != KEY_ESC && i) account.uin = i;
 		    break;
+
 		case 5:
-		    account.password = face.inputstr(spname + _(" password: "),
-			account.password, '*');
+		    tmp = face.inputstr(spname + _(" password: "), account.password, '*');
+		    if(face.getlastinputkey() != KEY_ESC && !tmp.empty())
+			account.password = tmp;
 		    break;
+
 		case 6:
 		    if(!account.empty()) {
 			face.status(_("Drop the account information first!"));
@@ -131,6 +138,7 @@ void accountmanager::exec() {
 			imcontrol.registration(account);
 		    }
 		    break;
+
 		case 7:
 		    imcontrol.updateinfo(account);
 		    break;
@@ -150,7 +158,7 @@ void accountmanager::exec() {
 
 		    tmp = face.inputstr(spname + _(" server address: "), tmp);
 
-		    if(!tmp.empty()) {
+		    if(face.getlastinputkey() != KEY_ESC && !tmp.empty()) {
 			if((pos = tmp.find(":")) != -1) {
 			    account.server = tmp.substr(0, pos);
 			    account.port = strtoul(tmp.substr(pos+1).c_str(), 0, 0);
@@ -171,9 +179,20 @@ void accountmanager::exec() {
 		case 11:
 		    imcontrol.channels(account);
 		    break;
+
+		case 12:
+		    if(!hook.online()) {
+			face.status(_("You must be logged in to preform the synchronization"));
+		    } else {
+			imcontrol.synclist(account);
+			fin = true;
+		    }
+		    break;
 	    }
 
-	    conf.setourid(account);
+	    if(action != 7) {
+		conf.setourid(account);
+	    }
 	}
     }
 
