@@ -1,7 +1,7 @@
 /*
 *
 * centericq core routines
-* $Id: centericq.cc,v 1.103 2002/06/27 13:56:44 konst Exp $
+* $Id: centericq.cc,v 1.114 2002/08/19 08:25:17 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -122,7 +122,7 @@ bool centericq::checkpasswords() {
 		    if(ia.password.empty()) {
 			r = false;
 			break;
-		    } else {
+		    } else if(face.getlastinputkey() != KEY_ESC) {
 			conf.setourid(ia);
 		    }
 		}
@@ -256,7 +256,7 @@ void centericq::mainloop() {
 
 	    case ACT_RENAME:
 		text = face.inputstr(_("New nickname to show: "), c->getdispnick());
-		if(text.size()) {
+		if(face.getlastinputkey() != KEY_ESC) {
 		    c->setdispnick(text);
 		    face.update();
 		}
@@ -577,7 +577,22 @@ void centericq::rereadstatus() {
 	ia = conf.getourid(pname);
 
 	if(!ia.empty()) {
-	    gethook(pname).setstatus(conf.getstatus(pname));
+	    char cst;
+	    imstatus st;
+	    string fname = conf.getconfigfname((string) "status-" + conf.getprotocolname(pname));
+	    ifstream f(fname.c_str());
+
+	    if(f.is_open()) {
+		f >> cst, f.close(), f.clear();
+		unlink(fname.c_str());
+
+		for(st = offline; st != imstatus_size; (int) st += 1) {
+		    if(imstatus2char[st] == cst) {
+			gethook(pname).setstatus(st);
+			break;
+		    }
+		}
+	    }
 	}
     }
 }
@@ -683,7 +698,7 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 	if(proceed) {
 	    if(proceed = face.eventedit(*sendev))
 	    if(proceed = !sendev->empty()) {
-		for(i = face.muins.begin(); i != face.muins.end(); i++) {
+		for(i = face.muins.begin(); i != face.muins.end(); ++i) {
 		    sendev->setcontact(*i);
 		    em.store(*sendev);
 		}
@@ -702,11 +717,54 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
     return proceed;
 }
 
-void centericq::readevents(const imcontact &cont) {
+void centericq::readevent(const imevent &ev, bool &enough, bool &fin) {
+    icqface::eventviewresult r = face.eventview(&ev);
+
+    switch(r) {
+	case icqface::forward:
+	    sendevent(ev, r);
+	    break;
+
+	case icqface::reply:
+	    sendevent(ev, r);
+	    enough = true;
+	    break;
+
+	case icqface::ok:
+	    enough = true;
+	    break;
+
+	case icqface::cancel:
+	    fin = true;
+	    break;
+
+	case icqface::open:
+	    if(const imurl *m = static_cast<const imurl *>(&ev))
+		conf.openurl(m->geturl());
+	    break;
+
+	case icqface::accept:
+	    em.store(imauthorization(ev.getcontact(),
+		imevent::outgoing, imauthorization::Granted, "accepted"));
+	    enough = true;
+	    break;
+
+	case icqface::reject:
+	    em.store(imauthorization(ev.getcontact(),
+		imevent::outgoing, imauthorization::Rejected, "rejected"));
+	    enough = true;
+	    break;
+
+	case icqface::info:
+	    cicq.userinfo(ev.getcontact());
+	    break;
+    }
+}
+
+void centericq::readevents(const imcontact cont) {
     vector<imevent *> events;
     vector<imevent *>::iterator iev;
     icqcontact *c = clist.get(cont);
-    icqface::eventviewresult r;
     bool fin, enough;
 
     if(c) {
@@ -716,48 +774,12 @@ void centericq::readevents(const imcontact &cont) {
 	    events = em.getevents(cont, c->getlastread());
 	    fin = events.empty();
 
-	    for(iev = events.begin(); (iev != events.end()) && !fin; iev++) {
+	    for(iev = events.begin(); (iev != events.end()) && !fin; ++iev) {
 		if((*iev)->getdirection() == imevent::incoming) {
 		    enough = false;
 
 		    while(!enough && !fin) {
-			r = face.eventview(*iev);
-
-			switch(r) {
-			    case icqface::forward:
-				sendevent(**iev, r);
-				break;
-
-			    case icqface::reply:
-				sendevent(**iev, r);
-				enough = true;
-				break;
-
-			    case icqface::ok:
-				enough = true;
-				break;
-
-			    case icqface::cancel:
-				fin = true;
-				break;
-
-			    case icqface::open:
-				if(const imurl *m = static_cast<const imurl *>(*iev))
-				    conf.openurl(m->geturl());
-				break;
-
-			    case icqface::accept:
-				em.store(imauthorization(cont,
-				    imevent::outgoing, true, "accepted"));
-				enough = true;
-				break;
-
-			    case icqface::reject:
-				em.store(imauthorization(cont,
-				    imevent::outgoing, false, "rejected"));
-				enough = true;
-				break;
-			}
+			readevent(**iev, enough, fin);
 		    }
 		}
 
@@ -804,13 +826,13 @@ void centericq::history(const imcontact &cont) {
 		    case icqface::accept:
 			enough = true;
 			em.store(imauthorization(cont, imevent::outgoing,
-			    true, "accepted"));
+			    imauthorization::Granted, "accepted"));
 			break;
 
 		    case icqface::reject:
 			enough = true;
 			em.store(imauthorization(cont, imevent::outgoing,
-			    false, "rejected"));
+			    imauthorization::Rejected, "rejected"));
 			break;
 
 		    case icqface::cancel:
@@ -832,6 +854,10 @@ void centericq::history(const imcontact &cont) {
 			    im = *i;
 			}
 			break;
+
+		    case icqface::info:
+			cicq.userinfo(cont);
+			break;
 		}
 	    }
 	}
@@ -855,7 +881,7 @@ string centericq::quotemsg(const string &text) {
 
     breakintolines(text, lines, 50/*face.sizeWArea.x2-face.sizeWArea.x1-4*/);
 
-    for(i = lines.begin(); i != lines.end(); i++) {
+    for(i = lines.begin(); i != lines.end(); ++i) {
 	if(!i->empty()) ret += (string) "> " + *i;
 	ret = trailcut(ret);
 	ret += "\n";
@@ -880,16 +906,6 @@ icqcontact *centericq::addcontact(const imcontact &ic) {
 	if(!groupid) return 0;
     }
 
-    abstracthook &hook = gethook(ic.pname);
-
-    if(hook.getcapabilities() & hoptCanNotify) {
-	if(face.ask(_("Notify the user he/she has been added?"),
-	ASK_YES | ASK_NO, ASK_YES) == ASK_YES) {
-//            ihook.sendalert(ic);
-	    face.log(_("+ the notification has been sent to %lu"), ic.uin);
-	}
-    }
-
     if(!c) {
 	c = clist.addnew(ic, false);
     } else {
@@ -1042,14 +1058,14 @@ void centericq::exectimers() {
 
 	    static map<protocolname, reconnectInfo> reconnect;
 
-	    if(!hook.logged()) {
+	    if(timer_current-reconnect[pname].timer > reconnect[pname].period) {
 		/*
 		*
 		* Any need to try auto re-connecting?
 		*
 		*/
 
-		if(timer_current-reconnect[pname].timer > reconnect[pname].period) {
+		if(!hook.logged()) {
 		    time(&reconnect[pname].timer);
 		    reconnect[pname].period += reconnect[pname].period/2;
 
@@ -1062,10 +1078,10 @@ void centericq::exectimers() {
 			}
 
 		    }
+		} else {
+		    fonline = true;
+		    reconnect[pname] = reconnectInfo();
 		}
-	    } else {
-		fonline = true;
-		reconnect[pname] = reconnectInfo();
 	    }
 	}
     }
@@ -1232,7 +1248,7 @@ string rusconv(const string &tdir, const string &text) {
     if(tdir == "wk") table = wk;
 
     if(conf.getrussian() && table) {
-	for(i = text.begin(); i != text.end(); i++) {
+	for(i = text.begin(); i != text.end(); ++i) {
 	    c = (unsigned char) *i;
 	    c &= 0377;
 	    if(c & 0200) c = table[c & 0177];
@@ -1257,9 +1273,7 @@ string rusconv(const string &tdir, const string &text) {
 }
 
 bool ischannel(const imcontact &cont) {
-    return (cont.nickname.substr(0, 1) == "#") && (cont.pname == irc);
-}
-
-bool ischannel(const icqcontact *c) {
-    return c ? ischannel(c->getdesc()) : false;
+    return
+	(cont.nickname.substr(0, 1) == "#") &&
+	(cont.pname == irc || cont.pname == yahoo);
 }
