@@ -64,7 +64,9 @@ class centericq {
 	bool idle(int options = 0);
 	void exectimers();
 
-	void readevents(const imcontact &cont);
+	void readevent(const imevent &ev, bool &enough, bool &fin);
+	void readevents(const imcontact cont);
+
 	void history(const imcontact &cont);
 
 	bool sendevent(const imevent &ev, icqface::eventviewresult r);
@@ -77,6 +79,5 @@ string rushtmlconv(const string &tdir, const string &text);
 string ruscrlfconv(const string &tdir, const string &text);
 
 bool ischannel(const imcontact &cont);
-bool ischannel(const icqcontact *c);
 
 #endif
