@@ -3,13 +3,14 @@
 #include "icqhook.h"
 #include "yahoohook.h"
 #include "msnhook.h"
+#include "aimhook.h"
+#include "irchook.h"
 
 #include <time.h>
 
-abstracthook::abstracthook() {
-    fcapabilities = 0;
-    searchdest = 0;
-}
+abstracthook::abstracthook()
+    : fcapabilities(0), searchdest(0)
+{ }
 
 void abstracthook::init() {
 }
@@ -71,25 +72,25 @@ bool abstracthook::send(const imevent &ev) {
     return false;
 }
 
-void abstracthook::sendnewuser(const imcontact c) {
+void abstracthook::sendnewuser(const imcontact &c) {
 }
 
 imstatus abstracthook::getstatus() const {
     return offline;
 }
 
-bool abstracthook::isdirectopen(const imcontact c) const {
+bool abstracthook::isdirectopen(const imcontact &c) const {
     return false;
 }
 
-void abstracthook::removeuser(const imcontact ic) {
+void abstracthook::removeuser(const imcontact &ic) {
 }
 
 int abstracthook::getcapabilities() const {
     return fcapabilities;
 }
 
-void abstracthook::requestinfo(const imcontact c) {
+void abstracthook::requestinfo(const imcontact &c) {
 }
 
 void abstracthook::lookup(const imsearchparams &params, verticalmenu &dest) {
@@ -105,18 +106,17 @@ void abstracthook::requestawaymsg(const imcontact &c) {
 // ----------------------------------------------------------------------------
 
 abstracthook &gethook(protocolname pname) {
-    static abstracthook ahook;
+    static abstracthook abshook;
 
     switch(pname) {
-	case icq:
-	    return ihook;
-	case yahoo:
-	    return yhook;
-	case msn:
-	    return mhook;
+	case icq: return ihook;
+	case yahoo: return yhook;
+	case msn: return mhook;
+	case aim: return ahook;
+	case irc: return irhook;
     }
 
-    return ahook;
+    return abshook;
 }
 
 struct tm *maketm(int hour, int minute, int day, int month, int year) {
