@@ -1,7 +1,7 @@
 /*
 *
 * centericq icq protocol handling class
-* $Id: icqhook.cc,v 1.86 2002/07/03 15:31:05 konst Exp $
+* $Id: icqhook.cc,v 1.99 2002/08/17 14:08:40 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -32,6 +32,9 @@
 #include <libicq2000/userinfohelpers.h>
 
 #define PERIOD_ICQPOLL  5
+#define PERIOD_RESOLVE  25
+
+#define MAX_REQUESTS    20
 
 icqhook ihook;
 
@@ -48,6 +51,7 @@ static const Status stat2int[imstatus_size] = {
 
 icqhook::icqhook() {
     fonline = false;
+    blockmode = Normal;
 
     fcapabilities =
 	hoptCanSendURL |
@@ -57,8 +61,11 @@ icqhook::icqhook() {
 	hoptCanChangeNick |
 	hoptCanUpdateDetails |
 	hoptChangableServer |
+	hoptCanSyncList |
 	hoptControlableVisibility;
 
+    cli.setServerSideGroup("centericq", 0x0666);
+
     cli.connected.connect(slot(this, &icqhook::connected_cb));
     cli.disconnected.connect(slot(this, &icqhook::disconnected_cb));
     cli.socket.connect(slot(this, &icqhook::socket_cb));
@@ -108,10 +115,14 @@ void icqhook::connect() {
     int i;
     icqcontact *c;
 
+    if(acc.additional["autosync"].empty()) acc.additional["autosync"] = "1";
+    if(acc.additional["webaware"].empty()) acc.additional["webaware"] = "1";
+    conf.setourid(acc);
+
     for(i = 0; i < clist.count; i++) {
 	c = (icqcontact *) clist.at(i);
 
-	if(c->getdesc().pname == icq) {
+	if(c->getdesc().pname == icq && c->getdesc().uin) {
 	    cli.addContact(new Contact(c->getdesc().uin));
 	}
     }
@@ -128,6 +139,7 @@ void icqhook::connect() {
 
     cli.setUIN(acc.uin);
     cli.setPassword(acc.password);
+    cli.setWebAware(acc.additional["webaware"] == "1");
 
     sendinvisible();
     cli.setStatus(stat2int[manualstatus], manualstatus == invisible);
@@ -143,17 +155,16 @@ void icqhook::disconnect() {
 }
 
 void icqhook::resolve() {
-    int i;
+    int i, cnt;
     icqcontact *c;
-    imcontact cont;
 
-    for(i = 0; i < clist.count; i++) {
+    for(i = cnt = 0; i < clist.count && cnt < MAX_REQUESTS; i++) {
 	c = (icqcontact *) clist.at(i);
-	cont = c->getdesc();
 
-	if(cont.pname == icq)
+	if(c->getdesc().pname == icq)
 	if(c->getdispnick() == i2str(c->getdesc().uin)) {
-	    requestinfo(cont);
+	    requestinfo(c);
+	    cnt++;
 	}
     }
 }
@@ -163,7 +174,7 @@ void icqhook::sendinvisible() {
     vector<visInfo>::iterator iv, it;
     icqlist::iterator i;
 
-    for(i = lst.begin(); i != lst.end(); i++) {
+    for(i = lst.begin(); i != lst.end(); ++i) {
 	if(i->getdesc().pname == icq) {
 	    switch(i->getstatus()) {
 		case csvisible:
@@ -177,7 +188,7 @@ void icqhook::sendinvisible() {
 
     nvislist = todo;
 
-    for(iv = vislist.begin(); iv != vislist.end(); iv++) {
+    for(iv = vislist.begin(); iv != vislist.end(); ++iv) {
 	it = find(todo.begin(), todo.end(), *iv);
 
 	if(it != todo.end()) {
@@ -190,7 +201,7 @@ void icqhook::sendinvisible() {
 	}
     }
 
-    for(it = todo.begin(); it != todo.end(); it++) {
+    for(it = todo.begin(); it != todo.end(); ++it) {
 	switch(it->second) {
 	    case csvisible: cli.addVisible(new Contact(it->first)); break;
 	    case csinvisible: cli.addInvisible(new Contact(it->first)); break;
@@ -209,6 +220,11 @@ void icqhook::exectimers() {
 	    sendinvisible();
 	    time(&timer_poll);
 	}
+
+	if(tcurrent-timer_resolve > PERIOD_RESOLVE) {
+	    resolve();
+	    time(&timer_resolve);
+	}
     }
 }
 
@@ -225,37 +241,37 @@ void icqhook::main() {
     tv.tv_sec = tv.tv_usec = 0;
     hsock = 0;
 
-    for(i = rfds.begin(); i != rfds.end(); i++) {
+    for(i = rfds.begin(); i != rfds.end(); ++i) {
 	FD_SET(*i, &rs);
 	hsock = max(hsock, *i);
     }
 
-    for(i = wfds.begin(); i != wfds.end(); i++) {
+    for(i = wfds.begin(); i != wfds.end(); ++i) {
 	FD_SET(*i, &ws);
 	hsock = max(hsock, *i);
     }
 
-    for(i = efds.begin(); i != efds.end(); i++) {
+    for(i = efds.begin(); i != efds.end(); ++i) {
 	FD_SET(*i, &es);
 	hsock = max(hsock, *i);
     }
 
     if(select(hsock+1, &rs, &ws, &es, &tv) > 0) {
-	for(i = rfds.begin(); i != rfds.end(); i++) {
+	for(i = rfds.begin(); i != rfds.end(); ++i) {
 	    if(FD_ISSET(*i, &rs)) {
 		cli.socket_cb(*i, SocketEvent::READ);
 		break;
 	    }
 	}
 
-	for(i = wfds.begin(); i != wfds.end(); i++) {
+	for(i = wfds.begin(); i != wfds.end(); ++i) {
 	    if(FD_ISSET(*i, &ws)) {
 		cli.socket_cb(*i, SocketEvent::WRITE);
 		break;
 	    }
 	}
 
-	for(i = efds.begin(); i != efds.end(); i++) {
+	for(i = efds.begin(); i != efds.end(); ++i) {
 	    if(FD_ISSET(*i, &es)) {
 		cli.socket_cb(*i, SocketEvent::EXCEPTION);
 		break;
@@ -267,17 +283,17 @@ void icqhook::main() {
 void icqhook::getsockets(fd_set &rs, fd_set &ws, fd_set &es, int &hsocket) const {
     vector<int>::const_iterator i;
 
-    for(i = rfds.begin(); i != rfds.end(); i++) {
+    for(i = rfds.begin(); i != rfds.end(); ++i) {
 	hsocket = max(hsocket, *i);
 	FD_SET(*i, &rs);
     }
 
-    for(i = wfds.begin(); i != wfds.end(); i++) {
+    for(i = wfds.begin(); i != wfds.end(); ++i) {
 	hsocket = max(hsocket, *i);
 	FD_SET(*i, &ws);
     }
 
-    for(i = efds.begin(); i != efds.end(); i++) {
+    for(i = efds.begin(); i != efds.end(); ++i) {
 	hsocket = max(hsocket, *i);
 	FD_SET(*i, &es);
     }
@@ -286,15 +302,15 @@ void icqhook::getsockets(fd_set &rs, fd_set &ws, fd_set &es, int &hsocket) const
 bool icqhook::isoursocket(fd_set &rs, fd_set &ws, fd_set &es) const {
     vector<int>::const_iterator i;
 
-    for(i = rfds.begin(); i != rfds.end(); i++)
+    for(i = rfds.begin(); i != rfds.end(); ++i)
 	if(FD_ISSET(*i, &rs))
 	    return true;
 
-    for(i = wfds.begin(); i != wfds.end(); i++)
+    for(i = wfds.begin(); i != wfds.end(); ++i)
 	if(FD_ISSET(*i, &ws))
 	    return true;
 
-    for(i = efds.begin(); i != efds.end(); i++)
+    for(i = efds.begin(); i != efds.end(); ++i)
 	if(FD_ISSET(*i, &es))
 	    return true;
 
@@ -322,6 +338,7 @@ bool icqhook::send(const imevent &ev) {
     ContactRef ic = cli.getContact(uin);
     MessageEvent *sev = 0;
     ICQMessageEvent *iev;
+    icqcontact *c;
 
     if(!ic.get()) {
 	return false;
@@ -355,7 +372,23 @@ bool icqhook::send(const imevent &ev) {
 
     } else if(ev.gettype() == imevent::authorization) {
 	const imauthorization *m = static_cast<const imauthorization *> (&ev);
-	sev = new AuthAckEvent(ic, ruscrlfconv("kw", m->gettext()), m->getgranted());
+
+	switch(m->getauthtype()) {
+	    case imauthorization::Granted:
+	    case imauthorization::Rejected:
+		sev = new AuthAckEvent(ic, ruscrlfconv("kw", m->getmessage()),
+		    m->getauthtype() == imauthorization::Granted);
+		break;
+
+	    case imauthorization::Request:
+		sev = new AuthReqEvent(ic, ruscrlfconv("kw", m->getmessage()));
+		if(c = clist.get(ev.getcontact())) {
+		    icqcontact::basicinfo bi = c->getbasicinfo();
+		    bi.authawait = true;
+		    c->setbasicinfo(bi);
+		}
+		break;
+	}
 
     } else {
 	return false;
@@ -373,13 +406,35 @@ bool icqhook::send(const imevent &ev) {
 }
 
 void icqhook::sendnewuser(const imcontact &c) {
-    if(logged()) {
+    if(logged() && c.uin) {
 	cli.addContact(new Contact(c.uin));
+	cli.fetchSimpleContactInfo(cli.getContact(c.uin));
 	cli.fetchDetailContactInfo(cli.getContact(c.uin));
+
+	if(conf.getourid(icq).additional["autosync"] == "1") {
+	    icqcontact *cc = clist.get(c);
+	    if(cc)
+	    if(cc->inlist())
+		cli.uploadServerBasedContactList(cli.getContact(c.uin));
+	}
     }
 }
 
 void icqhook::removeuser(const imcontact &c) {
+    if(conf.getourid(icq).additional["autosync"] == "1")
+    if(cli.getContact(c.uin).get()) {
+	icqcontact *cc = clist.get(c);
+	if(cc)
+	if(cc->inlist()) {
+	    ContactRef ic = cli.getContact(c.uin);
+	    if(ic->getServerBased()) {
+		ic->setAlias(cc->getnick());
+		ic->setAuthAwait(cc->getbasicinfo().authawait);
+		cli.removeServerBasedContactList(ic);
+	    }
+	}
+    }
+
     cli.removeContact(c.uin);
 }
 
@@ -475,7 +530,7 @@ imstatus icqhook::icq2imstatus(const Status st) const {
 }
 
 void icqhook::requestinfo(const imcontact &c) {
-    if(logged()) {
+    if(logged() && c.uin) {
 	if(c == imcontact(conf.getourid(icq).uin, icq)) {
 	    // Our info is requested
 	    cli.fetchSelfDetailContactInfo();
@@ -485,6 +540,7 @@ void icqhook::requestinfo(const imcontact &c) {
 	    if(!icont.get()) {
 		cli.addContact(new Contact(c.uin));
 	    } else {
+		cli.fetchSimpleContactInfo(icont);
 		cli.fetchDetailContactInfo(icont);
 	    }
 	}
@@ -505,7 +561,13 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	params.gender == genderFemale ? ICQ2000::SEX_FEMALE :
 	ICQ2000::SEX_UNSPECIFIED;
 
-    if(!params.email.empty() || params.minage || params.maxage ||
+    if(!params.kwords.empty()) {
+	searchevent = cli.searchForContacts(params.kwords);
+
+    } else if(params.randomgroup) {
+	searchevent = cli.searchForContacts((ICQ2000::RandomChatGroup) params.randomgroup);
+
+    } else if(!params.email.empty() || params.minage || params.maxage ||
     !params.city.empty() || !params.state.empty() ||
     !params.company.empty() || !params.department.empty() ||
     !params.position.empty() || params.onlineonly ||
@@ -516,9 +578,11 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	    params.language, params.city, params.state, params.country,
 	    params.company, params.department, params.position,
 	    params.onlineonly);
+
     } else {
 	searchevent = cli.searchForContacts(params.nick, params.firstname,
 	    params.lastname);
+
     }
 }
 
@@ -590,6 +654,16 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     ic->setMainHomeInfo(home);
     ic->setHomepageInfo(hpage);
     ic->setWorkInfo(work);
+    ic->setAuthReq(cbinfo.requiresauth);
+
+    icqconf::imaccount acc = conf.getourid(icq);
+    acc.additional["webaware"] = cbinfo.webaware ? "1" : "0";
+    acc.additional["randomgroup"] = i2str(cbinfo.randomgroup);
+    acc.additional["autosync"] = cbinfo.autosync ? "1" : "0";
+    conf.setourid(acc);
+
+    cli.setWebAware(cbinfo.webaware);
+    cli.setRandomChatGroup(cbinfo.randomgroup);
 
     cli.uploadSelfDetails();
     cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
@@ -604,6 +678,14 @@ void icqhook::requestawaymsg(const imcontact &c) {
 }
 
 void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
+    string sbuf;
+    vector<string> pintinfo, backginfo;
+    list<Contact::PersonalInterestInfo::Interest>::iterator ii;
+    list<Contact::BackgroundInfo::School>::iterator isc;
+    icqcontact::basicinfo cbinfo;
+    icqcontact::moreinfo cminfo;
+    icqcontact::workinfo cwinfo;
+
     if(ic.get() && c) {
 	Contact::MainHomeInfo &home = ic->getMainHomeInfo();
 	Contact::HomepageInfo &hpage = ic->getHomepageInfo();
@@ -612,9 +694,9 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 	Contact::BackgroundInfo &backg = ic->getBackgroundInfo();
 	Contact::EmailInfo &email = ic->getEmailInfo();
 
-	icqcontact::basicinfo cbinfo = c->getbasicinfo();
-	icqcontact::moreinfo cminfo = c->getmoreinfo();
-	icqcontact::workinfo cwinfo = c->getworkinfo();
+	cbinfo = c->getbasicinfo();
+	cminfo = c->getmoreinfo();
+	cwinfo = c->getworkinfo();
 
 	/* basic information */
 
@@ -669,11 +751,7 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 
 	/* personal interests */
 
-	string sbuf;
-	vector<string> pintinfo;
-	list<Contact::PersonalInterestInfo::Interest>::iterator ii;
-
-	for(ii = pint.interests.begin(); ii != pint.interests.end(); ii++) {
+	for(ii = pint.interests.begin(); ii != pint.interests.end(); ++ii) {
 	    sbuf = UserInfoHelpers::getInterestsIDtoString(ii->first);
 
 	    if(!ii->second.empty()) {
@@ -686,10 +764,7 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 
 	/* education background */
 
-	vector<string> backginfo;
-	list<Contact::BackgroundInfo::School>::iterator isc;
-
-	for(isc = backg.schools.begin(); isc != backg.schools.end(); isc++) {
+	for(isc = backg.schools.begin(); isc != backg.schools.end(); ++isc) {
 	    sbuf = UserInfoHelpers::getBackgroundIDtoString(isc->first);
 	    if(!sbuf.empty()) sbuf += ": ";
 	    sbuf += rusconv("wk", isc->second);
@@ -717,14 +792,157 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
     }
 }
 
+void icqhook::processemailevent(const string &sender, const string &email, const string &message) {
+    icqcontact *c = clist.getemail(email);
+
+    if(!c)
+    if(c = clist.addnew(imcontact(0, infocard), true)) {
+	c->setdispnick(email);
+	c->setnick(email);
+
+	icqcontact::basicinfo b = c->getbasicinfo();
+
+	if(sender != email) {
+	    int pos = sender.find(" ");
+
+	    if(pos != -1) {
+		b.fname = sender.substr(0, pos);
+		b.lname = sender.substr(pos+1);
+	    } else {
+		b.fname = sender;
+	    }
+	}
+
+	b.email = email;
+	c->setbasicinfo(b);
+    }
+
+    if(c) {
+	em.store(imemail(c, imevent::incoming,
+	    rusconv("wk", message)));
+    }
+}
+
+void icqhook::synclist() {
+    fd_set srfds, swfds, sefds;
+    struct timeval tv;
+    int hsockfd, s;
+    time_t t = time(0);
+
+    blockmode = SyncList;
+    syncstatus = reqUpload;
+
+    /*
+    *
+    * Let's now make a list of contacts to be uploaded.
+    *
+    */
+
+    ContactList tcl;
+    vector<icqcontact *> tobestored, needauth;
+    vector<icqcontact *>::iterator ic;
+
+    getsyncstatus(s, tobestored, needauth);
+    for(ic = tobestored.begin(); ic != tobestored.end(); ++ic) {
+	ContactRef ct = cli.getContact((*ic)->getdesc().uin);
+	if(ct.get()) {
+	    ct->setAlias((*ic)->getnick());
+	    tcl.add(ct);
+	}
+    }
+
+    cli.uploadServerBasedContactList(tcl);
+
+    while(syncstatus != ackFetch && logged() && time(0)-t <= 30) {
+	hsockfd = 0;
+
+	FD_ZERO(&srfds);
+	FD_ZERO(&swfds);
+	FD_ZERO(&sefds);
+
+	getsockets(srfds, swfds, sefds, hsockfd);
+
+	tv.tv_sec = 10;
+	tv.tv_usec = 0;
+
+	select(hsockfd+1, &srfds, &swfds, &sefds, &tv);
+
+	if(isoursocket(srfds, swfds, sefds)) {
+	    main();
+
+	    switch(syncstatus) {
+		case ackUpload:
+		    syncstatus = reqFetch;
+		    cli.fetchServerBasedContactList();
+		    break;
+	    }
+	}
+    }
+
+    blockmode = Normal;
+}
+
+void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored, vector<icqcontact *> &needauth) {
+    ContactList &cl = cli.getContactList();
+    ContactList::iterator ic = cl.begin();
+
+    tobestored.clear();
+    synchronized = 0;
+
+    while(ic != cl.end()) {
+	imcontact imc = imcontact((*ic)->getUIN(), icq);
+	icqcontact *c = clist.get(imc);
+
+	if(c) {
+	    if(!(*ic)->getServerBased()) {
+		if(!(*ic)->getAuthAwait()) {
+		    tobestored.push_back(c);
+
+		} else if(!c->getbasicinfo().authawait) {
+		    if(find(needauth.begin(), needauth.end(), c) == needauth.end())
+			needauth.push_back(c);
+
+		}
+	    } else {
+		synchronized++;
+
+	    }
+	}
+
+	++ic;
+    }
+}
+
+void icqhook::sendaddauth() {
+    int synchronized;
+    ContactList tcl;
+    ContactRef ct;
+    vector<icqcontact *> tobestored, needauth;
+    vector<icqcontact *>::const_iterator in;
+
+    getsyncstatus(synchronized, tobestored, needauth);
+
+    for(in = needauth.begin(); in != needauth.end(); ++in) {
+	ct = cli.getContact((*in)->getdesc().uin);
+	ct->setAuthAwait(false);
+	tcl.add(ct);
+    }
+
+    cli.uploadServerBasedContactList(tcl);
+}
+
 // ----------------------------------------------------------------------------
 
 void icqhook::connected_cb(ConnectedEvent *ev) {
     flogged = true;
 
     time(&timer_poll);
+    time(&timer_resolve);
+
     logger.putourstatus(icq, offline, manualstatus);
 
+    cli.setRandomChatGroup(strtoul(conf.getourid(icq).additional["randomgroup"].c_str(), 0, 0));
+
     face.log(_("+ [icq] logged in"));
     face.update();
 
@@ -744,8 +962,8 @@ void icqhook::connected_cb(ConnectedEvent *ev) {
 	unlink(conf.getconfigfname("icq-infoset").c_str());
     }
 
-    resolve();
-//    cli.fetchServerBasedContactList();
+    if(conf.getourid(icq).additional["autosync"] == "1")
+	cli.fetchServerBasedContactList();
 }
 
 void icqhook::disconnected_cb(DisconnectedEvent *ev) {
@@ -824,7 +1042,7 @@ void icqhook::messaged_cb(MessageEvent *ev) {
 	    }
 
 	    if(c) {
-		em.store(imsms(c->getdesc(), imevent::incoming,
+		em.store(imsms(c, imevent::incoming,
 		    rusconv("wk", r->getMessage())));
 	    }
 	}
@@ -832,49 +1050,49 @@ void icqhook::messaged_cb(MessageEvent *ev) {
     } else if(ev->getType() == MessageEvent::AuthReq) {
 	AuthReqEvent *r;
 	if(r = dynamic_cast<AuthReqEvent *>(ev)) {
-	    em.store(imauthorization(ic, imevent::incoming, false,
+	    em.store(imauthorization(ic, imevent::incoming, imauthorization::Request,
 		rusconv("wk", r->getMessage())));
 	}
 
+    } else if(ev->getType() == MessageEvent::AuthAck) {
+	AuthAckEvent *r;
+	if(r = dynamic_cast<AuthAckEvent *>(ev)) {
+	    icqcontact *c = clist.get(ic);
+	    if(c) {
+		icqcontact::basicinfo bi = c->getbasicinfo();
+		bi.authawait = false;
+		c->setbasicinfo(bi);
+	    }
+
+	    if(r->isGranted()) {
+		em.store(imnotification(ic, _("The user has accepted your authorization request")));
+
+	    } else {
+		em.store(imnotification(ic, (string)
+		    _("The user has rejected your authorization request; the message was: ") + r->getMessage()));
+
+	    }
+	}
+
     } else if(ev->getType() == MessageEvent::EmailEx) {
 	EmailExEvent *r;
 
 	if(r = dynamic_cast<EmailExEvent *>(ev)) {
-	    icqcontact *c = clist.getemail(r->getEmail());
+	    processemailevent(r->getSender(), r->getEmail(), r->getMessage());
+	}
 
-	    if(!c)
-	    if(c = clist.addnew(imcontact(0, infocard), true)) {
-		c->setdispnick(r->getEmail());
-		c->setnick(r->getEmail());
+    } else if(ev->getType() == MessageEvent::WebPager) {
+	WebPagerEvent *r;
 
-		icqcontact::basicinfo b = c->getbasicinfo();
-
-		if(r->getSender() != r->getEmail()) {
-		    int pos = r->getSender().find(" ");
-
-		    if(pos != -1) {
-			b.fname = r->getSender().substr(0, pos);
-			b.lname = r->getSender().substr(pos+1);
-		    } else {
-			b.fname = r->getSender();
-		    }
-		}
-
-		b.email = r->getEmail();
-		c->setbasicinfo(b);
-	    }
-
-	    if(c) {
-		em.store(imemail(c->getdesc(), imevent::incoming,
-		    rusconv("wk", r->getMessage())));
-	    }
+	if(r = dynamic_cast<WebPagerEvent *>(ev)) {
+	    processemailevent(r->getSender(), r->getEmail(), r->getMessage());
 	}
 
     } else if(ev->getType() == MessageEvent::UserAdd) {
 	UserAddEvent *r;
 	if(r = dynamic_cast<UserAddEvent *>(ev)) {
-	    em.store(immessage(ic, imevent::incoming,
-		string() + _("* The user has added you to his/her contact list")));
+	    em.store(imnotification(ic,
+		_("The user has added you to his/her contact list")));
 	}
 
     }
@@ -915,8 +1133,7 @@ void icqhook::messageack_cb(MessageEvent *ev) {
 		    AwayMessageEvent *r;
 
 		    if(r = dynamic_cast<AwayMessageEvent *>(ev)) {
-			em.store(immessage(ic, imevent::incoming,
-			    string() + _("* Away message:") + "\n\n" +
+			em.store(imnotification(ic, string() + _("Away message:") + "\n\n" +
 			    rusconv("wk", r->getAwayMessage())));
 		    }
 		} else {
@@ -941,13 +1158,13 @@ void icqhook::contact_status_change_signal_cb(StatusChangeEvent *ev) {
     icqcontact *c = clist.get(imcontact(ev->getUIN(), icq));
 
     if(c) {
-	ContactRef ic = cli.getContact(ev->getUIN());
+	ContactRef ic = ev->getContact();
 	StatusChangeEvent *tev = dynamic_cast<StatusChangeEvent*>(ev);
 	imstatus nst = icq2imstatus(tev->getStatus());
 
 	if(ic->isInvisible()) nst = invisible;
 
-	logger.putonline(c->getdesc(), c->getstatus(), nst);
+	logger.putonline(c, c->getstatus(), nst);
 	c->setstatus(nst);
 
 	if(c->getstatus() != offline)
@@ -957,7 +1174,7 @@ void icqhook::contact_status_change_signal_cb(StatusChangeEvent *ev) {
 
 void icqhook::contact_userinfo_change_signal_cb(UserInfoChangeEvent *ev) {
     icqcontact *c = clist.get(imcontact(ev->getUIN(), icq));
-    ContactRef ic = cli.getContact(ev->getUIN());
+    ContactRef ic = ev->getContact();
 
     if(!c) {
 	c = clist.get(contactroot);
@@ -1067,12 +1284,18 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 	    icqcontact *c = new icqcontact(imcontact(cont->getUIN(), icq));
 	    icqcontact::basicinfo binfo = c->getbasicinfo();
 
-	    c->setnick(rusconv("wk", cont->getAlias()));
-	    c->setdispnick(c->getnick());
+	    if(ev->getSearchType() == SearchResultEvent::RandomChat) {
+		binfo.fname = _("Random Chat User");
 
-	    binfo.fname = rusconv("wk", cont->getFirstName());
-	    binfo.lname = rusconv("wk", cont->getLastName());
-	    binfo.email = rusconv("wk", cont->getEmail());
+	    } else {
+		c->setnick(rusconv("wk", cont->getAlias()));
+		c->setdispnick(c->getnick());
+
+		binfo.fname = rusconv("wk", cont->getFirstName());
+		binfo.lname = rusconv("wk", cont->getLastName());
+		binfo.email = rusconv("wk", cont->getEmail());
+
+	    }
 
 	    c->setbasicinfo(binfo);
 
@@ -1102,8 +1325,23 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 
 void icqhook::self_contact_userinfo_change_cb(UserInfoChangeEvent *ev) {
     if(!ev->isTransientDetail()) {
-	updateinforecord(cli.getSelfContact(), clist.get(contactroot));
+	icqcontact *c = clist.get(contactroot);
+
+	updateinforecord(cli.getSelfContact(), c);
 	updateinforecord(cli.getSelfContact(), clist.get(imcontact(cli.getUIN(), icq)));
+
+	/*
+	*
+	* Fill in the locally stored parts now.
+	*
+	*/
+
+	icqconf::imaccount im = conf.getourid(icq);
+	icqcontact::basicinfo cbinfo = c->getbasicinfo();
+	cbinfo.webaware = im.additional["webaware"] == "1";
+	cbinfo.randomgroup = strtoul(im.additional["randomgroup"].c_str(), 0, 0);
+	cbinfo.autosync = im.additional["autosync"] == "1";
+	c->setbasicinfo(cbinfo);
     }
 }
 
@@ -1114,15 +1352,57 @@ void icqhook::self_contact_status_change_cb(StatusChangeEvent *ev) {
 void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
     const ContactList &lst = ev->getContactList();
     ContactList::const_iterator i = lst.begin();
+    icqcontact *c;
 
-    while(i != lst.end()) {
-	imcontact cont((*i)->getUIN(), icq);
+    if(ev->getType() == ServerBasedContactEvent::Fetch) {
+	syncstatus = ackFetch;
 
-	if(!clist.get(cont))
-	    clist.addnew(cont, true);
+	while(i != lst.end()) {
+	    cli.addContact(*i);
+	    imcontact cont((*i)->getUIN(), icq);
+	    if(!clist.get(cont)) clist.addnew(cont, false);
+	    ++i;
+	}
+
+    } else if(ev->getType() == ServerBasedContactEvent::Upload) {
+	syncstatus = ackUpload;
+
+	map<unsigned int, ServerBasedContactEvent::UploadResult> r = ev->getUploadResults();
+
+	while(i != lst.end()) {
+	    if(r[(*i)->getUIN()] == ServerBasedContactEvent::AuthRequired) {
+		c = clist.get(imcontact((*i)->getUIN(), icq));
+
+		if(c) {
+		    icqcontact::basicinfo bi = c->getbasicinfo();
+
+		    if(blockmode == Normal) {
+			if(!bi.authawait) {
+			    face.log(_("+ [icq] auto-requesting authorization to store %lu"), (*i)->getUIN());
+			    em.store(imauthorization(c->getdesc(), imevent::outgoing, imauthorization::Request));
+			    cli.uploadServerBasedContactList(*i);
+			}
+
+		    } else if(blockmode == SyncList) {
+			ContactRef ct = cli.getContact((*i)->getUIN());
+			if(ct.get()) {
+			    ct->setAuthAwait(true);
+			    bi.authawait = false;
+			    c->setbasicinfo(bi);
+			    // So that it doesn't get into the next upload
+			    // packet sent..
+			}
+
+		    }
+		}
+	    }
+	    ++i;
+	}
+
+    } else if(ev->getType() == ServerBasedContactEvent::Remove) {
+	while(i != lst.end()) {
+	    ++i;
+	}
 
-	i++;
     }
-
-//    cli.uploadServerBasedContactList();
 }
