@@ -16,7 +16,19 @@ class icqhook: public abstracthook, public SigC::Object {
 
 	vector<int> rfds, wfds, efds;
 
-	time_t timer_poll;
+	enum {
+	    reqUpload,
+	    ackUpload,
+	    reqFetch,
+	    ackFetch
+	} syncstatus;
+
+	enum {
+	    Normal,
+	    SyncList
+	} blockmode;
+
+	time_t timer_poll, timer_resolve;
 	bool fonline, flogged;
 	unsigned int reguin;
 	SearchResultEvent *searchevent;
@@ -47,6 +59,7 @@ class icqhook: public abstracthook, public SigC::Object {
 	void resolve();
 	void sendinvisible();
 	void updateinforecord(ContactRef ic, icqcontact *c);
+	void processemailevent(const string &sender, const string &email, const string &message);
 
     public:
 	icqhook();
@@ -83,6 +96,10 @@ class icqhook: public abstracthook, public SigC::Object {
 
 	void lookup(const imsearchparams &params, verticalmenu &dest);
 	void sendupdateuserinfo(const icqcontact &c);
+
+	void synclist();
+	void getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored, vector<icqcontact *> &needauth);
+	void sendaddauth();
 };
 
 extern icqhook ihook;
