@@ -1,7 +1,7 @@
 /*
 *
 * centericq IRC protocol handling class
-* $Id: irchook.cc,v 1.28 2002/07/03 15:31:06 konst Exp $
+* $Id: irchook.cc,v 1.40 2002/08/16 16:48:28 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -78,6 +78,7 @@ void irchook::init() {
     firetalk_register_callback(handle, FC_CHAT_LEFT, &chatleft);
     firetalk_register_callback(handle, FC_CHAT_KICKED, &chatkicked);
     firetalk_register_callback(handle, FC_ERROR, &errorhandler);
+    firetalk_register_callback(handle, FC_IM_USER_NICKCHANGED, &nickchanged);
 
 #ifdef DEBUG
     firetalk_register_callback(handle, FC_LOG, &log);
@@ -221,12 +222,13 @@ void irchook::removeuser(const imcontact &ic) {
     if(online()) {
 	if(!ischannel(ic)) {
 	    firetalk_im_remove_buddy(handle, ic.nickname.c_str());
-	} else {
-	    vector<channelInfo>::iterator i;
-	    i = find(channels.begin(), channels.end(), ic.nickname);
-	    if(i != channels.end()) i->contactlist = false;
 	}
     }
+
+    if(ischannel(ic)) {
+	vector<channelInfo>::iterator i = find(channels.begin(), channels.end(), ic.nickname);
+	if(i != channels.end()) i->contactlist = false;
+    }
 }
 
 void irchook::setautostatus(imstatus st) {
@@ -288,6 +290,7 @@ void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
 
     emailsub = params.email;
     namesub = params.firstname;
+    searchsincelast = params.sincelast && !params.room.empty();
 
     searchchannels.clear();
     extlisted.clear();
@@ -346,7 +349,8 @@ void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
 void irchook::processnicks() {
     string dnick;
     char *nick;
-    vector<string> foundnicks;
+    vector<string> foundnicks, prelastfound;
+    static vector<string> lastfoundnicks;
     vector<string>::iterator in, isn;
     vector<channelInfo>::iterator ir;
     bool remove;
@@ -366,7 +370,7 @@ void irchook::processnicks() {
 	for(in = foundnicks.begin(); in != foundnicks.end(); ) {
 	    remove = false;
 
-	    for(ir = channels.begin(); !remove && (ir != channels.end()); ir++) {
+	    for(ir = channels.begin(); !remove && (ir != channels.end()); ++ir) {
 		if(find(searchchannels.begin(), searchchannels.end(), ir->name) == searchchannels.end())
 		    continue;
 		if(ir->name == *searchchannels.begin())
@@ -392,7 +396,22 @@ void irchook::processnicks() {
 	}
     }
 
-    for(in = foundnicks.begin(); in != foundnicks.end(); in++) {
+    prelastfound = foundnicks;
+
+    if(searchsincelast) {
+	for(in = foundnicks.begin(); in != foundnicks.end(); ) {
+	    if(find(lastfoundnicks.begin(), lastfoundnicks.end(), *in) != lastfoundnicks.end()) {
+		foundnicks.erase(in);
+		in = foundnicks.begin();
+	    } else {
+		in++;
+	    }
+	}
+    }
+
+    lastfoundnicks = prelastfound;
+
+    for(in = foundnicks.begin(); in != foundnicks.end(); ++in) {
 	dnick = *in;
 
 	npos = 0;
@@ -422,7 +441,7 @@ vector<irchook::channelInfo> irchook::getautochannels() const {
     vector<channelInfo> r;
     vector<channelInfo>::const_iterator ic;
 
-    for(ic = channels.begin(); ic != channels.end(); ic++) {
+    for(ic = channels.begin(); ic != channels.end(); ++ic) {
 	if(find(searchchannels.begin(), searchchannels.end(), ic->name) != searchchannels.end())
 	    if(!ic->joined)
 		// A channel used for search
@@ -444,7 +463,7 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
     *
     */
 
-    for(ic = channels.begin(); ic != channels.end(); ic++) {
+    for(ic = channels.begin(); ic != channels.end(); ++ic) {
 	iac = find(achannels.begin(), achannels.end(), ic->name);
 
 	if(ic->joined) {
@@ -467,7 +486,7 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
     *
     */
 
-    for(iac = achannels.begin(); iac != achannels.end(); iac++) {
+    for(iac = achannels.begin(); iac != achannels.end(); ++iac) {
 	ic = find(channels.begin(), channels.end(), iac->name);
 
 	if(iac->joined) {
@@ -481,7 +500,7 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
 	    if(!r) r = !ic->contactlist;
 	    if(r) {
 		icqcontact *c = clist.addnew(imcontact(iac->name, irc), false);
-		c->setstatus(iac->joined ? available : offline);
+		c->setstatus((iac->joined && gethook(irc).logged()) ? available : offline);
 	    }
 	}
     }
@@ -491,8 +510,8 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
 }
 
 void irchook::requestawaymsg(const imcontact &ic) {
-    em.store(immessage(ic, imevent::incoming, string() +
-	_("* Away message:") + "\n\n" + awaymessages[ic.nickname]));
+    em.store(imnotification(ic, string() +
+	_("Away message:") + "\n\n" + awaymessages[ic.nickname]));
 }
 
 void irchook::saveconfig() const {
@@ -503,10 +522,9 @@ void irchook::saveconfig() const {
     if(f.is_open()) {
 	f << "name\t" << ircname << endl;
 
-	for(ic = savech.begin(); ic != savech.end(); ic++)
+	for(ic = savech.begin(); ic != savech.end(); ++ic)
 	    f << "channel\t" << ic->name
 		<< "\t" << (ic->joined ? "1" : "0")
-		<< "\t" << (ic->contactlist ? "1" : "0")
 		<< endl;
 
 	f.close();
@@ -530,7 +548,7 @@ void irchook::loadconfig() {
 	    if(param == "channel") {
 		channels.push_back(channelInfo(getword(buf)));
 		channels.back().joined = (getword(buf) == "1");
-		channels.back().contactlist = (buf == "1");
+		channels.back().contactlist = clist.get(imcontact(channels.back().name, irc));
 	    }
 	}
 
@@ -554,6 +572,31 @@ void irchook::rawcommand(const string &cmd) {
     }
 }
 
+void irchook::channelfatal(string room, const char *fmt, ...) {
+    va_list ap;
+    char buf[1024];
+    vector<channelInfo>::iterator i;
+
+    va_start(ap, fmt);
+    vsprintf(buf, fmt, ap);
+    va_end(ap);
+
+    if(room.substr(0, 1) != "#")
+	room.insert(0, "#");
+
+    i = find(channels.begin(), channels.end(), room);
+
+    if(i != channels.end()) {
+	imcontact cont(room, irc);
+	icqcontact *c = clist.get(cont);
+	if(!c) c = clist.addnew(cont);
+	c->setstatus(offline);
+	i->joined = i->fetched = false;
+	i->contactlist = true;
+	em.store(imnotification(cont, buf));
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 void irchook::userstatus(const string &nickname, imstatus st) {
@@ -590,7 +633,7 @@ void irchook::connected(void *conn, void *cli, ...) {
 	}
     }
 
-    for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ic++) {
+    for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic) {
 	if(ic->joined) {
 	    firetalk_chat_join(irhook.handle, ic->name.c_str());
 	}
@@ -608,7 +651,7 @@ void irchook::disconnected(void *conn, void *cli, ...) {
     clist.setoffline(irc);
 
     vector<channelInfo>::iterator ic;
-    for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ic++)
+    for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic)
 	ic->fetched = false;
 
     face.log(_("+ [irc] disconnected from the network"));
@@ -846,7 +889,7 @@ void irchook::chatnames(void *connection, void *cli, ...) {
     }
 
     if(irhook.searchdest && irhook.emailsub.empty() && irhook.namesub.empty()) {
-	for(is = irhook.searchchannels.begin(); is != irhook.searchchannels.end(); is++) {
+	for(is = irhook.searchchannels.begin(); is != irhook.searchchannels.end(); ++is) {
 	    ic = find(irhook.channels.begin(), irhook.channels.end(), *is);
 
 	    if(ic != irhook.channels.end())
@@ -927,7 +970,7 @@ void irchook::endextended(void *connection, void *cli, ...) {
 	    if(!ic->joined)
 		firetalk_chat_part(irhook.handle, irhook.extlisted.back().c_str());
 
-	    for(is = irhook.searchchannels.begin(); ready && is != irhook.searchchannels.end(); is++)
+	    for(is = irhook.searchchannels.begin(); ready && is != irhook.searchchannels.end(); ++is)
 		ready = find(irhook.extlisted.begin(), irhook.extlisted.end(), *is) != irhook.extlisted.end();
 	}
     }
@@ -994,7 +1037,7 @@ void irchook::chatkicked(void *connection, void *cli, ...) {
     char *reason = va_arg(ap, char *);
     va_end(ap);
 
-    irhook.channelfatal(room, _("* Kicked by %s; reason: %s"),
+    irhook.channelfatal(room, _("Kicked by %s; reason: %s"),
 	by, rushtmlconv("wk", reason).c_str());
 }
 
@@ -1013,31 +1056,45 @@ void irchook::errorhandler(void *connection, void *cli, ...) {
 	    // Cannot join channel
 	    if(subject)
 	    if(strlen(subject))
-		irhook.channelfatal(subject, _("* %s"), description);
+		irhook.channelfatal(subject, "%s", description);
+	    break;
+	case FE_BADUSER:
+	    // Cannot fetch user's info
+	    if(subject)
+	    if(strlen(subject))
+	    if(c = clist.get(imcontact(subject, irc)))
+		c->setstatus(offline);
 	    break;
     }
 }
 
-void irchook::channelfatal(const string &room, const char *fmt, ...) {
+void irchook::nickchanged(void *connection, void *cli, ...) {
     va_list ap;
-    char buf[1024];
-    vector<channelInfo>::iterator i;
+    icqcontact *c;
+    char buf[100];
 
-    va_start(ap, fmt);
-    vsprintf(buf, fmt, ap);
+    va_start(ap, cli);
+    char *oldnick = va_arg(ap, char *);
+    char *newnick = va_arg(ap, char *);
     va_end(ap);
 
-    i = find(channels.begin(), channels.end(),
-	(string) (room.substr(0, 1) != "#" ? "#" : "") + room);
+    if(oldnick && newnick)
+    if(strcmp(oldnick, newnick)) {
+	if(c = clist.get(imcontact(oldnick, irc))) {
+	    if(!clist.get(imcontact(newnick, irc))) {
+		if(!c->inlist()) {
+		    c->setnick(newnick);
+		    if(c->getdispnick() == oldnick) c->setdispnick(newnick);
+		}
 
-    if(i != channels.end()) {
-	imcontact cont(room, irc);
-	icqcontact *c = clist.get(cont);
-	if(!c) c = clist.addnew(cont);
-	c->setstatus(offline);
-	i->joined = i->fetched = false;
-	i->contactlist = true;
-	em.store(immessage(cont, imevent::incoming, buf));
+	    } else {
+		c->setstatus(offline);
+
+	    }
+
+	    sprintf(buf, _("The user has changed their nick from %s to %s"), oldnick, newnick);
+	    em.store(imnotification(c, buf));
+	}
     }
 }
 
