@@ -1,7 +1,7 @@
 /*
 *
 * centericq configuration handling routines
-* $Id: icqconf.cc,v 1.70 2002/05/14 07:31:06 konst Exp $
+* $Id: icqconf.cc,v 1.78 2002/08/19 08:25:17 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -26,6 +26,10 @@
 #include <dirent.h>
 #include <fstream>
 
+#ifdef __sun__
+#include <sys/statvfs.h>
+#endif
+
 #include "icqconf.h"
 #include "icqface.h"
 #include "icqcontacts.h"
@@ -68,7 +72,7 @@ void icqconf::setourid(const imaccount &im) {
 	int port;
     } defservers[protocolname_size] = {
 	{ "login.icq.com", 5190 },
-	{ "", 0 },
+	{ "scs.yahoo.com", 5050 },
 	{ "messenger.hotmail.com", 1863 },
 	{ "toc.oscar.aol.com", 9898 },
 	{ "irc.openprojects.net", 6667 }
@@ -167,7 +171,7 @@ void icqconf::loadmainconfig() {
 
     if(f.is_open()) {
 	mailcheck = askaway = false;
-	savepwd = true;
+	savepwd = bidi = true;
 	setsmtphost("");
 
 	while(!f.eof()) {
@@ -190,6 +194,7 @@ void icqconf::loadmainconfig() {
 	    if(param == "smtp") setsmtphost(buf); else
 	    if(param == "log") makelog = true; else
 	    if(param == "chatmode") chatmode = true; else
+	    if(param == "nobidi") setbidi(false); else
 	    if(param == "askaway") askaway = true; else {
 		for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 		    buf = getprotocolname(pname);
@@ -233,6 +238,7 @@ void icqconf::save() {
 	    if(getmailcheck()) f << "mailcheck" << endl;
 	    if(getaskaway()) f << "askaway" << endl;
 	    if(getchatmode()) f << "chatmode" << endl;
+	    if(!getbidi()) f << "nobidi" << endl;
 
 	    f << "smtp\t" << getsmtphost() << ":" << dec << getsmtpport() << endl;
 
@@ -244,7 +250,7 @@ void icqconf::save() {
 	    if(getmakelog()) f << "log" << endl;
 
 	    vector<imaccount>::iterator ia;
-	    for(ia = accounts.begin(); ia != accounts.end(); ia++)
+	    for(ia = accounts.begin(); ia != accounts.end(); ++ia)
 		ia->write(f);
 
 	    f.close();
@@ -339,10 +345,10 @@ void icqconf::loadsounds() {
 	    fo << "# *\tmeans default sound for all the contacts" << endl;
 	    fo << "# icq_<uin>\tfor icq contacts" << endl;
 	    fo << "# yahoo_<nickname>\tfor yahoo" << endl;
-	    fo << "# msn_<nickname>\tmsn contacts" << endl << "#" << endl;
+	    fo << "# .. etc. Similar for the other protocols" << endl << "#" << endl;
 
 	    fo << "# <event>\tcan be: ";
-	    for(isn = soundnames.begin(); isn != soundnames.end(); isn++) {
+	    for(isn = soundnames.begin(); isn != soundnames.end(); ++isn) {
 		if(isn != soundnames.begin()) fo << ", ";
 		fo << isn->second;
 	    }
@@ -381,7 +387,7 @@ void icqconf::loadsounds() {
 
 	    it = imevent::imeventtype_size;
 
-	    for(isn = soundnames.begin(); isn != soundnames.end(); isn++) {
+	    for(isn = soundnames.begin(); isn != soundnames.end(); ++isn) {
 		if(skey == isn->second) {
 		    it = isn->first;
 		    break;
@@ -391,14 +397,24 @@ void icqconf::loadsounds() {
 	    if(it == imevent::imeventtype_size)
 		continue;
 
+	    c = 0;
+
 	    if(suin != "*") {
 		if((i = suin.find("_")) != -1) {
 		    skey = suin.substr(0, i);
 		    suin.erase(0, i+1);
 
-		    if(skey == "icq") c = clist.get(imcontact(strtoul(suin.c_str(), 0, 0), icq)); else
-		    if(skey == "yahoo") c = clist.get(imcontact(suin, yahoo)); else
-		    if(skey == "msn") c = clist.get(imcontact(suin, msn));
+		    imcontact ic;
+		    protocolname pname;
+
+		    for(pname = icq; pname != protocolname_size && skey != getprotocolname(pname); (int) pname += 1);
+
+		    if(pname != protocolname_size) {
+			if(pname == icq) ic = imcontact(strtoul(suin.c_str(), 0, 0), pname);
+			else ic = imcontact(suin, pname);
+
+			c = clist.get(ic);
+		    }
 		}
 	    } else {
 		c = clist.get(contactroot);
@@ -610,33 +626,24 @@ string icqconf::getprotocolprefix(protocolname pname) const {
 }
 
 imstatus icqconf::getstatus(protocolname pname) {
-    string fname, buf;
-    ifstream f;
     imstatus st = available;
+    map<string, string>::iterator ia;
+    imaccount a = getourid(pname);
 
-    fname = getconfigfname("status-" + getprotocolname(pname));
-    f.open(fname.c_str());
-
-    if(f.is_open()) {
-	getstring(f, buf);
-	st = (imstatus) ((int) atol(buf.c_str()));
-	f.close();
+    if((ia = a.additional.find("status")) != a.additional.end()) {
+	if(!ia->second.empty()) {
+	    for(st = offline; st != imstatus_size && imstatus2char[st] != ia->second[0]; (int) st += 1);
+	    if(st == imstatus_size) st = available;
+	}
     }
 
     return st;
 }
 
 void icqconf::savestatus(protocolname pname, imstatus st) {
-    string fname;
-    ofstream f;
-
-    fname = getconfigfname("status-" + getprotocolname(pname));
-    f.open(fname.c_str());
-
-    if(f.is_open()) {
-	f << (int) st << endl;
-	f.close();
-    }
+    imaccount im = getourid(pname);
+    im.additional["status"] = string(1, imstatus2char[st]);
+    setourid(im);
 }
 
 int icqconf::getprotcolor(protocolname pname) const {
@@ -655,6 +662,13 @@ void icqconf::setrussian(bool frussian) {
     russian = frussian;
 }
 
+void icqconf::setbidi(bool fbidi) {
+    bidi = fbidi;
+#ifdef USE_FRIBIDI
+    use_fribidi = bidi;
+#endif
+}
+
 void icqconf::commandline(int argc, char **argv) {
     int i;
     string event, proto, dest;
@@ -805,7 +819,10 @@ void icqconf::externalstatuschange(char st, const string &proto) const {
 			f.close();
 
 			if(rpid > 0) {
-			    conf.savestatus(pname, imst);
+			    string sfname = conf.getconfigfname((string) "status-" + proto);
+			    ofstream sf(sfname.c_str());
+			    if(sf.is_open()) sf << imstatus2char[imst], sf.close();
+
 			    kill(rpid, SIGUSR1);
 			}
 		    }
@@ -879,11 +896,15 @@ unsigned int icqconf::getsmtpport() const {
 }
 
 void icqconf::checkdiskspace() {
-    struct statfs st;
-
     fenoughdiskspace = true;
 
+#ifndef __sun__
+    struct statfs st;
     if(!statfs(conf.getdirname().c_str(), &st)) {
+#else
+    struct statvfs st;
+    if(!statvfs(conf.getdirname().c_str(), &st)) {
+#endif
 	fenoughdiskspace = ((double) st.f_bavail) * st.f_bsize >= 10240;
     }
 }
@@ -906,6 +927,7 @@ bool icqconf::imaccount::empty() const {
 
 void icqconf::imaccount::write(ofstream &f) {
     string prefix = conf.getprotocolname(pname) + "_";
+    map<string, string>::iterator ia;
 
     f << endl;
     if(!nickname.empty()) f << prefix << "nick\t" << nickname << endl;
@@ -916,6 +938,10 @@ void icqconf::imaccount::write(ofstream &f) {
 	if(port) f << ":" << port;
 	f << endl;
     }
+
+    for(ia = additional.begin(); ia != additional.end(); ++ia) {
+	f << prefix << ia->first << "\t" << ia->second << endl;
+    }
 }
 
 void icqconf::imaccount::read(const string &spec) {
@@ -940,6 +966,8 @@ void icqconf::imaccount::read(const string &spec) {
 	    if(pname == icq)
 	    if(server == "icq.mirabilis.com")
 		server = "";
+	} else {
+	    additional[spname] = buf;
 	}
     }
 }
