@@ -1,7 +1,7 @@
 /*
 *
 * centericq single icq contact class
-* $Id: icqcontact.cc,v 1.55 2002/05/26 07:17:32 konst Exp $
+* $Id: icqcontact.cc,v 1.60 2002/08/16 16:48:26 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -78,7 +78,7 @@ string icqcontact::tosane(const string &p) const {
     string buf;
     string::iterator i;
 
-    for(buf = p, i = buf.begin(); i != buf.end(); i++) {
+    for(buf = p, i = buf.begin(); i != buf.end(); ++i) {
 	if(strchr("\n\r", *i)) *i = ' ';
     }
 
@@ -150,11 +150,15 @@ void icqcontact::save() {
 	    tname = getdirname() + "info";
 	    f.open(tname.c_str());
 	    if(f.is_open()) {
+		string options;
+		if(binfo.requiresauth) options += "a";
+		if(binfo.authawait) options += "w";
+
 		f << nick << endl <<
 		    tosane(binfo.fname) << endl <<
 		    tosane(binfo.lname) << endl <<
 		    tosane(binfo.email) << endl <<
-		    endl <<
+		    options << endl <<
 		    endl <<
 		    tosane(binfo.city) << endl <<
 		    tosane(binfo.state) << endl <<
@@ -243,7 +247,10 @@ void icqcontact::load() {
 		case  1: binfo.fname = buf; break;
 		case  2: binfo.lname = buf; break;
 		case  3: binfo.email = buf; break;
-		case  4: break;
+		case  4:
+		    binfo.requiresauth = (strstr(buf, "a") != 0);
+		    binfo.authawait = (strstr(buf, "w") != 0);
+		    break;
 		case  5: break;
 		case  6: binfo.city = buf; break;
 		case  7: binfo.state = buf; break;
@@ -375,6 +382,7 @@ void icqcontact::excludefromlist() {
 }
 
 void icqcontact::includeintolist() {
+    gethook(cdesc.pname).sendnewuser(cdesc);
     unlink((getdirname() + "excluded").c_str());
     finlist = true;
 }
@@ -438,7 +446,15 @@ void icqcontact::setstatus(imstatus fstatus) {
 }
 
 void icqcontact::setnick(const string &fnick) {
+    string dir;
+
     nick = fnick;
+
+    if(!cdesc.nickname.empty()) {
+	dir = getdirname();
+	cdesc.nickname = fnick;
+	rename(dir.c_str(), getdirname().c_str());
+    }
 }
 
 void icqcontact::setdispnick(const string &fnick) {
@@ -647,6 +663,7 @@ string icqcontact::getpostponed() const {
 
 void icqcontact::setgroupid(int agroupid) {
     groupid = agroupid;
+    save();
 }
 
 int icqcontact::getgroupid() const {
@@ -676,9 +693,7 @@ void icqcontact::remindbirthday(bool r) {
 	    f.open(tname.c_str());
 	    if(f.is_open()) f.close();
 
-	    em.store(immessage(getdesc(), imevent::incoming,
-		string() + _("* The user has a birthday today")));
-
+	    em.store(imnotification(getdesc(), _("The user has a birthday today")));
 	    congratulated = true;
 	}
 
