@@ -23,12 +23,14 @@ enum imgender {
 class icqcontact {
     public:
 	struct basicinfo {
-	    basicinfo(): country(0), requiresauth(false) {};
+	    basicinfo()
+		: country(0), requiresauth(false), webaware(false),
+		  authawait(false), autosync(true), randomgroup(0) { };
 
 	    string fname, lname, email, city, state, phone, fax;
 	    string street, cellular, zip;
-	    unsigned short country;
-	    bool requiresauth;
+	    unsigned short country, randomgroup;
+	    bool requiresauth, webaware, authawait, autosync;
 	};
 
 	struct moreinfo {
