@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class, dialogs related part
-* $Id: icqdialogs.cc,v 1.75 2002/07/03 14:35:20 konst Exp $
+* $Id: icqdialogs.cc,v 1.83 2002/08/16 16:48:26 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -82,7 +82,7 @@ bool icqface::sprofmanager(string &name, string &act) {
     while(!finished) {
 	m.clear();
 
-	for(ip = profiles.begin(); ip != profiles.end(); ip++) {
+	for(ip = profiles.begin(); ip != profiles.end(); ++ip) {
 	    m.additemf(0, (void *) ip->first.c_str(), " %s: %s",
 		conf.getprotocolname(ip->second.pname).c_str(),
 		ip->first.c_str());
@@ -119,7 +119,7 @@ bool icqface::sprofmanager(string &name, string &act) {
 
     unlink(conf.getconfigfname("search-profiles").c_str());
 
-    for(ip = profiles.begin(); ip != profiles.end(); ip++) {
+    for(ip = profiles.begin(); ip != profiles.end(); ++ip) {
 	ip->second.save(ip->first);
     }
 
@@ -184,7 +184,7 @@ bool icqface::finddialog(imsearchparams &s) {
 		i = tree.addnode(_(" UIN "));
 		tree.addleaff(i, 0, 10, " %s ", strint(s.uin));
 
-		if(!s.uin) {
+		if(!s.uin && !s.randomgroup && s.kwords.empty()) {
 		    i = tree.addnode(_(" Details "));
 		    tree.addleaff(i, 0, 11, _(" Nickname : %s "), s.nick.c_str());
 		    tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
@@ -210,6 +210,16 @@ bool icqface::finddialog(imsearchparams &s) {
 		    i = tree.addnode(_(" Online only "));
 		    tree.addleaff(i, 0, 25, " %s ", stryesno[s.onlineonly]);
 		}
+
+		if(!s.uin && s.kwords.empty()) {
+		    i = tree.addnode(_(" Random chat group "));
+		    tree.addleaff(i, 0, 28, " %s ", strrandomgroup[s.randomgroup]);
+		} 
+
+		if(!s.uin && !s.randomgroup) {
+		    i = tree.addnode(_(" Keywords "));
+		    tree.addleaff(i, 0, 29, " %s ", s.kwords.c_str());
+		}
 		break;
 
 	    default:
@@ -227,6 +237,11 @@ bool icqface::finddialog(imsearchparams &s) {
 	    }
 
 	    tree.addleaff(i, 0, 12, _(" E-Mail : %s "), s.email.c_str());
+
+	    if(!s.room.empty()) {
+		i = tree.addnode(_(" Joined since the last check only "));
+		tree.addleaff(i, 0, 16, " %s ", stryesno[s.sincelast]);
+	    }
 	}
 
 	finished = !db.open(n, b, (void **) &i);
@@ -277,6 +292,7 @@ bool icqface::finddialog(imsearchparams &s) {
 		    case 13: s.firstname = inputstr(_("First name: "), s.firstname); break;
 		    case 14: s.lastname = inputstr(_("Last name: "), s.lastname); break;
 		    case 15: selectagerange(s.agerange); break;
+		    case 16: s.sincelast = !s.sincelast; break;
 		    case 17: selectgender(s.gender); break;
 		    case 18: selectlanguage(s.language); break;
 		    case 19: s.city = inputstr(_("City: "), s.city); break;
@@ -288,6 +304,8 @@ bool icqface::finddialog(imsearchparams &s) {
 		    case 25: s.onlineonly = !s.onlineonly; break;
 		    case 26: s.room = inputstr(_("Channel: "), s.room); break;
 		    case 27: s.firstname = inputstr(_("Name: "), s.firstname); break;
+		    case 28: selectrandomgroup(s.randomgroup); break;
+		    case 29: s.kwords = inputstr(_("Keywords: "), s.kwords); break;
 		}
 		break;
 
@@ -375,6 +393,15 @@ void icqface::gendetails(treeview *tree, icqcontact *c) {
 	tree->addleaff(i, 0, 38, _(" 3rd language : %s "),
 	    ICQ2000::UserInfoHelpers::getLanguageIDtoString(mi.lang3).c_str());
 
+	i = tree->addnode(_(" Miscellaneous "));
+
+	if(ourdetails) {
+	    tree->addleaff(i, 0, 43, _(" Enable web status indicator : %s "), stryesno[bi.webaware]);
+	    tree->addleaff(i, 0, 42, _(" Random chat group : %s "), strrandomgroup[bi.randomgroup]);
+	    tree->addleaff(i, 0, 45, _(" Store contact list server-side : %s "), stryesno[bi.autosync]);
+	} else {
+	    tree->addleaff(i, 0, 44, _(" Authorization required : %s "), stryesno[bi.requiresauth]);
+	}
     }
 
     i = tree->addnode(_(" About "));
@@ -496,7 +523,6 @@ bool icqface::updatedetails(icqcontact *c, protocolname upname) {
 		case 26: wi.state = inputstr(_("State: "), wi.state); break;
 		case 27: selectcountry(wi.country); break;
 		case 28: wi.street = inputstr(_("Street address: "), wi.street); break;
-		case 41: wi.zip = inputstr(_("Zip code: "), wi.zip); break;
 		case 29: wi.company = inputstr(_("Company: "), wi.company); break;
 		case 30: wi.dept = inputstr(_("Department: "), wi.dept); break;
 		case 31: wi.position = inputstr(_("Position: "), wi.position); break;
@@ -512,6 +538,11 @@ bool icqface::updatedetails(icqcontact *c, protocolname upname) {
 
 		case 39: edit(about, _("About")); break;
 		case 40: bi.requiresauth = !bi.requiresauth; break;
+		case 41: wi.zip = inputstr(_("Zip code: "), wi.zip); break;
+		case 42: selectrandomgroup(bi.randomgroup); break;
+		case 43: bi.webaware = !bi.webaware; break;
+		case 44: bi.requiresauth = !bi.requiresauth; break;
+		case 45: bi.autosync = !bi.autosync; break;
 	    }
 
 	    c->setbasicinfo(bi);
@@ -570,6 +601,24 @@ void icqface::selectlanguage(unsigned short &f) {
     if(i) f = i-1;
 }
 
+void icqface::selectrandomgroup(unsigned short &f) {
+    int i;
+    verticalmenu m(conf.getcolor(cp_dialog_menu), conf.getcolor(cp_dialog_selected));
+    m.setwindow(textwindow(4, LINES-13, 20, LINES-3, conf.getcolor(cp_dialog_menu)));
+
+    m.additemf(" %s", _("none"));
+
+    for(i = 1; i <= ((int) ICQ2000::group_SeekingMen)+1; i++) {
+	m.additemf(" %s", strrandomgroup[i]);
+    }
+
+    m.setpos(f);
+    i = m.open();
+    m.close();
+
+    if(i) f = i-1;
+}
+
 void icqface::selectgender(imgender &f) {
     verticalmenu m(conf.getcolor(cp_dialog_menu), conf.getcolor(cp_dialog_selected));
     m.setwindow(textwindow(4, LINES-8, 20, LINES-4, conf.getcolor(cp_dialog_menu)));
@@ -722,6 +771,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     bool makelog = conf.getmakelog();
     bool askaway = conf.getaskaway();
     bool chatmode = conf.getchatmode();
+    bool bidi = conf.getbidi();
 
     icqconf::groupmode gmode = conf.getgroupmode();
 
@@ -756,10 +806,14 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
     while(!finished) {
 	t.clear();
 
-	i = t.addnode(_(" Sound and colors "));
+	i = t.addnode(_(" User interface "));
 	t.addleaff(i, 0, 1, _(" Change sound device to : %s "), strregsound(s));
 	t.addleaff(i, 0, 2, _(" Change color scheme to : %s "), strregcolor(c));
 
+#ifdef USE_FRIBIDI
+	t.addleaff(i, 0, 20, _( " Enable bidirectional languages support : %s "), stryesno[bidi]);
+#endif
+
 	i = t.addnode(_(" Contact list "));
 	t.addleaff(i, 0, 17, _(" Arrange contacts into groups : %s "), strgroupmode(gmode));
 	t.addleaff(i, 0,  6, _(" Hide offline users : %s "), stryesno[hideoffl]);
@@ -849,6 +903,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 			tmp = inputstr(_("SMTP server hostname: "), smtp);
 			if(!tmp.empty()) smtp = tmp;
 			break;
+		    case 20: bidi = !bidi; break;
 		}
 		break;
 	    case 1:
@@ -863,6 +918,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 		conf.setmakelog(makelog);
 		conf.setaskaway(askaway);
 		conf.setchatmode(chatmode);
+		conf.setbidi(bidi);
 
 		if(conf.getgroupmode() != gmode) {
 		    conf.setgroupmode(gmode);
@@ -871,6 +927,7 @@ bool icqface::updateconf(icqconf::regsound &s, icqconf::regcolor &c) {
 
 		conf.setsockshost(socks ? prserv : "");
 		conf.setsmtphost(smtp);
+		conf.save();
 		break;
 	}
     }
@@ -919,6 +976,7 @@ int icqface::groupmanager(const string &text, bool sel) {
     int r, ngrp, b, id;
     vector<icqgroup>::iterator i;
     string gname;
+    bool touched = false;
 
     textwindow w(0, 0, sizeDlg.width, sizeDlg.height,
 	conf.getcolor(cp_dialog_frame), TW_CENTERED);
@@ -953,7 +1011,7 @@ int icqface::groupmanager(const string &text, bool sel) {
 
 	sort(groups.begin(), groups.end());
 
-	for(i = groups.begin(); i != groups.end(); i++) {
+	for(i = groups.begin(); i != groups.end(); ++i) {
 	    b = i-groups.begin()+1;
 	    id = t.addleaff(ngrp, 0, b, " %s ", i->getname().c_str());
 	    if(n == b) t.setcur(id);
@@ -967,6 +1025,7 @@ int icqface::groupmanager(const string &text, bool sel) {
 		gname = inputstr(_("Name for a group to be created: "), "");
 		if(!gname.empty()) {
 		    groups.add(gname);
+		    touched = true;
 		}
 		break;
 
@@ -974,7 +1033,10 @@ int icqface::groupmanager(const string &text, bool sel) {
 		if(n) {
 		    i = groups.begin()+n-1;
 		    gname = inputstr(_("New name for the group: "), i->getname());
-		    if(!gname.empty()) i->rename(gname);
+		    if(!gname.empty()) {
+			i->rename(gname);
+			touched = true;
+		    }
 		}
 		break;
 
@@ -986,6 +1048,7 @@ int icqface::groupmanager(const string &text, bool sel) {
 			if(ask(_("Are you sure want to remove the group?"), ASK_YES | ASK_NO, ASK_NO) == ASK_YES) {
 			    groups.remove(i->getid());
 			    clist.rearrange();
+			    touched = true;
 			}
 		    }
 		}
@@ -996,6 +1059,7 @@ int icqface::groupmanager(const string &text, bool sel) {
 		    i = groups.begin()+n-1;
 		    i->moveup();
 		    n--;
+		    touched = true;
 		}
 		break;
 
@@ -1004,6 +1068,7 @@ int icqface::groupmanager(const string &text, bool sel) {
 		    i = groups.begin()+n-1;
 		    i->movedown();
 		    n++;
+		    touched = true;
 		}
 		break;
 
@@ -1018,6 +1083,8 @@ int icqface::groupmanager(const string &text, bool sel) {
 	}
     }
 
+    if(touched) groups.save();
+
     db.close();
     unblockmainscreen();
     return r;
