@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class
-* $Id: icqface.cc,v 1.121 2002/07/03 15:31:04 konst Exp $
+* $Id: icqface.cc,v 1.130 2002/08/16 16:48:27 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -541,9 +541,14 @@ void icqface::fillcontactlist() {
 		c->getmsgcount() ? conf.getcolor(cp_main_highlight) : conf.getprotcolor(c->getdesc().pname),
 		c, "%s%s ", c->getmsgcount() ? "#" : " ", dnick.c_str());
 	} else {
+	    char *fmt = "%s[%c] %s ";
+
+	    if(lst.inlist(c->getdesc(), csvisible)) fmt = "%s<%c> %s "; else
+	    if(lst.inlist(c->getdesc(), csinvisible)) fmt = "%s{%c} %s ";
+
 	    mcontacts->addleaff(nnode,
 		c->getmsgcount() ? conf.getcolor(cp_main_highlight) : conf.getprotcolor(c->getdesc().pname),
-		c, "%s[%c] %s ", c->getmsgcount() ? "#" : " ", c->getshortstatus(), dnick.c_str());
+		c, fmt, c->getmsgcount() ? "#" : " ", c->getshortstatus(), dnick.c_str());
 	}
 
 //        if(savec && c->getmsgcount() && ontop) savec = 0;
@@ -591,17 +596,17 @@ bool icqface::findresults(const imsearchparams &sp) {
 	conf.getcolor(cp_main_selected), _("Details"), _("Message"), _("Add"),
 	_("New search"), 0));
 
-    gethook(sp.pname).lookup(sp, *db.getmenu());
-
     db.addautokeys();
     db.redraw();
 
-    db.idle = &dialogidle;
-    db.otherkeys = &findreskeys;
-
     mainw.write(sizeWArea.x1+2, sizeWArea.y1,
 	conf.getcolor(cp_main_highlight), _("Searching contacts.."));
 
+    gethook(sp.pname).lookup(sp, *db.getmenu());
+
+    db.idle = &dialogidle;
+    db.otherkeys = &findreskeys;
+
     workarealine(sizeWArea.y1+2);
     workarealine(sizeWArea.y2-2);
 
@@ -723,7 +728,7 @@ void commaform(string &text) {
 }
 
 void icqface::infogeneral(dialogbox &db, icqcontact *c) {
-    string langs;
+    string langs, options;
     icqcontact::basicinfo bi = c->getbasicinfo();
     icqcontact::moreinfo mi = c->getmoreinfo();
 
@@ -870,7 +875,7 @@ void icqface::infointerests(dialogbox &db, icqcontact *c) {
     if(!data.empty()) {
 	text += (string) "* " + _("Interests") + "\n";
 
-	for(i = data.begin(); i != data.end(); i++) {
+	for(i = data.begin(); i != data.end(); ++i) {
 	    text += " + " + *i + "\n";
 	}
 
@@ -882,7 +887,7 @@ void icqface::infointerests(dialogbox &db, icqcontact *c) {
     if(!data.empty()) {
 	text += (string) "* " + _("Background") + "\n";
 
-	for(i = data.begin(); i != data.end(); i++)
+	for(i = data.begin(); i != data.end(); ++i)
 	    text += " + " + *i + "\n";
     }
 
@@ -1045,7 +1050,7 @@ bool icqface::changestatus(protocolname &pname, imstatus &st) {
 	    _(" [i] Invisible")
 	};
 
-	for(im = mst.begin(); im != mst.end(); im++) {
+	for(im = mst.begin(); im != mst.end(); ++im) {
 	    m.additem(0, *im, statustext[im-mst.begin()]);
 
 	    if(pname != proto_all)
@@ -1274,14 +1279,13 @@ void icqface::modelist(contactstatus cs) {
 		muins.clear();
 
 		if(multicontacts(_("Select contacts to add to the list"))) {
-		    for(ic = muins.begin(); ic != muins.end(); ic++) {
+		    for(ic = muins.begin(); ic != muins.end(); ++ic) {
 			lst.push_back(modelistitem(clist.get(*ic)->getdispnick(), *ic, cs));
 			if(cs == csignore) clist.remove(*ic);
 		    }
 
 		    lst.fillmenu(db.getmenu(), cs);
 		    db.getmenu()->redraw();
-		    face.update();
 		}
 		break;
 	    case 2:
@@ -1295,10 +1299,11 @@ void icqface::modelist(contactstatus cs) {
 		    lst.del(it.getdesc(), cs);
 		    lst.fillmenu(db.getmenu(), cs);
 		    db.getmenu()->redraw();
-		    face.update();
 		}
 		break;
 	}
+
+	face.relaxedupdate();
     }
 
     db.close();
@@ -1339,7 +1344,7 @@ bool icqface::multicontacts(const string &ahead ) {
 	m.getpos(saveelem, savefirst);
 	m.clear();
 
-	for(c = mlst.begin(); c != mlst.end(); c++) {
+	for(c = mlst.begin(); c != mlst.end(); ++c) {
 	    icqcontact *cont = (icqcontact *) clist.get(*c);
 
 	    m.additemf(conf.getprotcolor(c->pname), cont, " [%c] %s",
@@ -1490,7 +1495,7 @@ void icqface::quickfind(verticalmenu *multi ) {
 			nick += k;
 		    }
 
-		    for(is = nick.begin(), upnick = ""; is != nick.end(); is++)
+		    for(is = nick.begin(), upnick = ""; is != nick.end(); ++is)
 			upnick += toupper(*is);
 
 		    bool fin = false;
@@ -1524,7 +1529,7 @@ void icqface::quickfind(verticalmenu *multi ) {
 			    current.erase(len);
 
 			    for(is = current.begin(), upcurrent = "";
-			    is != current.end(); is++)
+			    is != current.end(); ++is)
 				    upcurrent += toupper(*is);
 
 			    if(upnick == upcurrent) {
@@ -1578,7 +1583,7 @@ void icqface::showextractedurls() {
 	    conf.getcolor(cp_main_highlight),
 	    _("URLs within the current context"));
 
-	for(i = extractedurls.begin(); i != extractedurls.end(); i++)
+	for(i = extractedurls.begin(); i != extractedurls.end(); ++i)
 	    m.additem(" " + *i);
 
 	if(n = m.open())
@@ -1670,6 +1675,7 @@ bool icqface::eventedit(imevent &ev) {
 
     }
 
+    editdone = false;
     restoreworkarea();
     status("");
     return r;
@@ -1679,7 +1685,7 @@ void icqface::renderchathistory() {
     int count, chatmargin;
     string text;
     char buf[64];
-    time_t t;
+    time_t t, lastread;
     struct stat st;
 
     vector<imevent *> events;
@@ -1690,6 +1696,7 @@ void icqface::renderchathistory() {
     histentry h;
 
     icqcontact *c = clist.get(passinfo);
+    if(!c) return;
 
     count = 0;
 
@@ -1703,6 +1710,7 @@ void icqface::renderchathistory() {
     count = 0;
     events = em.getevents(passinfo, chatlastread);
     chatlastread = 0;
+    lastread = c->getlastread();
 
     if(events.size()) {
 	c->setlastread(events.back()->gettimestamp());
@@ -1714,6 +1722,17 @@ void icqface::renderchathistory() {
     }
 
     while(events.size()) {
+	if(events.back()->getdirection() == imevent::incoming)
+	if(events.back()->gettimestamp() > lastread) {
+	    if(events.back()->gettype() == imevent::authorization) {
+		bool fin, enough;
+		fin = enough = false;
+
+		while(!fin && !enough)
+		    cicq.readevent(*events.back(), enough, fin);
+	    }
+	}
+
 	if(count < chatlines) {
 	    text = (string) time2str(&(t = events.back()->gettimestamp()), "DD.MM hh:mm", buf) + " ";
 	    text += events.back()->gettext();
@@ -1738,13 +1757,13 @@ void icqface::renderchathistory() {
     for(count = 0; count < chatlines; count++)
 	mvprintw(chatmargin-count, sizeWArea.x1+2, "%s", text.c_str());
 
-    for(count = 0, ir = toshow.begin(); ir != toshow.end() && count < chatlines; ir++) {
+    for(count = 0, ir = toshow.begin(); ir != toshow.end() && count < chatlines; ++ir) {
 	switch(ir->first) {
 	    case imevent::incoming: attrset(conf.getcolor(cp_main_text)); break;
 	    case imevent::outgoing: attrset(conf.getcolor(cp_main_highlight)); break;
 	}
 
-	for(il = ir->second.rbegin(); il != ir->second.rend() && count < chatlines; il++) {
+	for(il = ir->second.rbegin(); il != ir->second.rend() && count < chatlines; ++il) {
 	    kgotoxy(sizeWArea.x1+2, chatmargin-count);
 	    printstring(*il);
 	    count++;
@@ -1782,7 +1801,7 @@ void icqface::chat(const imcontact &ic) {
 
     status(_("Ctrl-X send, Ctrl-P multiple, Ctrl-O history, Alt-? details, ESC cancel"));
 
-    for(bool finished = false; !finished; ) {
+    for(bool finished = false; !finished && clist.get(ic); ) {
 	renderchathistory();
 
 	editdone = false;
@@ -1795,7 +1814,7 @@ void icqface::chat(const imcontact &ic) {
 	if(editdone) {
 	    auto_ptr<imevent> ev(new immessage(ic, imevent::outgoing, p.get()));
 
-	    for(i = muins.begin(); i != muins.end(); i++) {
+	    for(i = muins.begin(); i != muins.end(); ++i) {
 		ev.get()->setcontact(*i);
 		em.store(*ev.get());
 	    }
@@ -1827,7 +1846,7 @@ icqface::eventviewresult icqface::eventview(const imevent *ev, vector<eventviewr
     vector<eventviewresult> actions;
     vector<eventviewresult>::iterator ia;
 
-    if(ev->gettype() == imevent::message) {
+    if(ev->gettype() == imevent::message || ev->gettype() == imevent::notification) {
 	actions.push_back(forward);
 
 	if(ev->getdirection() == imevent::incoming) {
@@ -1855,6 +1874,7 @@ icqface::eventviewresult icqface::eventview(const imevent *ev, vector<eventviewr
 
     } else if(ev->gettype() == imevent::authorization) {
 	if(ev->getdirection() == imevent::incoming) {
+	    actions.push_back(info);
 	    actions.push_back(accept);
 	    actions.push_back(reject);
 	}
@@ -1885,7 +1905,7 @@ icqface::eventviewresult icqface::eventview(const imevent *ev, vector<eventviewr
 
     bar->item = actions.size()-1;
 
-    for(ia = actions.begin(); ia != actions.end(); ia++) {
+    for(ia = actions.begin(); ia != actions.end(); ++ia) {
 	bar->items.push_back(eventviewresultnames[*ia]);
     }
 
@@ -1939,7 +1959,7 @@ void icqface::histmake(const vector<imevent *> &hist) {
     mhist.clear();
     mhist.setpos(0);
 
-    for(i = hist.rbegin(); i != hist.rend(); i++) {
+    for(i = hist.rbegin(); i != hist.rend(); ++i) {
 	imevent &ev = **i;
 
 	color = 0;
@@ -2061,7 +2081,7 @@ void icqface::menuidle(verticalmenu &m) {
 	    flog = face.lastlog;
 	    face.lastlog.clear();
 
-	    for(il = flog.begin(); il != flog.end(); il++)
+	    for(il = flog.begin(); il != flog.end(); ++il)
 		face.log(*il);
 
 	    face.dotermresize = false;
@@ -2149,15 +2169,22 @@ int icqface::contactskeys(verticalmenu &m, int k) {
 
 	case 'r':
 	case 'R':
-	    if(!ischannel(c))
+	    if(!ischannel(c) && c)
 		face.extk = ACT_RENAME;
 	    break;
 
+#if 0
 	case 'c':
 	case 'C':
-//            face.extk = ACT_CONTACT;
+	    ihook.saveserverbasedcl();
 	    break;
-	
+
+	case 'd':
+	case 'D':
+	    ihook.readserverbasedcl();
+	    break;
+#endif
+
 	case ALT('s'):
 	case '/': face.extk = ACT_QUICKFIND; break;
     }
