@@ -1,7 +1,7 @@
 /*
 *
 * centericq messages history handling class
-* $Id: icqhist.cc,v 1.6 2001/08/17 19:11:59 konst Exp $
+* $Id: icqhist.cc,v 1.14 2002/08/16 16:48:27 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -33,20 +33,22 @@ icqhistory::icqhistory() {
 icqhistory::~icqhistory() {
 }
 
-FILE *icqhistory::open(unsigned int uin, const char *mode) {
-    icqcontact *c = clist.get(uin);
+FILE *icqhistory::open(const imcontact cinfo, const char *mode) {
+    icqcontact *c = clist.get(cinfo);
 
     if(!c) {
 	if(conf.getantispam()) return 0;
-	else clist.addnew(uin);
+	c = clist.addnew(cinfo);
     }
 
-    string fname = (string) getenv("HOME") + "/.centericq/" + i2str(uin) + "/history";
+    string fname = c->getdirname() + "/history";
+
     return fopen(fname.c_str(), mode);
 }
 
-void icqhistory::putmessage(unsigned int uin, string text, int dir, struct tm *timestamp) {
-    FILE *f = open(uin, "a");
+void icqhistory::putmessage(const imcontact cinfo, const string text,
+int dir, struct tm *timestamp) {
+    FILE *f = open(cinfo, "a");
     if(f) {
 	fprintf(f, "\f\n");
 	fprintf(f, "%s\nMSG\n", dir == HIST_MSG_OUT ? "OUT" : "IN");
@@ -55,8 +57,9 @@ void icqhistory::putmessage(unsigned int uin, string text, int dir, struct tm *t
     }
 }
 
-void icqhistory::puturl(unsigned int uin, string url, string desc, int dir, struct tm *timestamp) {
-    FILE *f = open(uin, "a");
+void icqhistory::puturl(const imcontact cinfo, const string url,
+const string desc, int dir, struct tm *timestamp) {
+    FILE *f = open(cinfo, "a");
     if(f) {
 	fprintf(f, "\f\n");
 	fprintf(f, "%s\nURL\n", dir == HIST_MSG_OUT ? "OUT" : "IN");
@@ -65,9 +68,9 @@ void icqhistory::puturl(unsigned int uin, string url, string desc, int dir, stru
     }
 }
 
-void icqhistory::putfile(unsigned int uin, unsigned long seq, string fname,
-int fsize, int dir, struct tm *timestamp) {
-    FILE *f = open(uin, "a");
+void icqhistory::putfile(const imcontact cinfo, unsigned long seq,
+const string fname, int fsize, int dir, struct tm *timestamp) {
+    FILE *f = open(cinfo, "a");
     if(f) {
 	fprintf(f, "\f\n");
 	fprintf(f, "%s\nFILE\n", dir == HIST_MSG_OUT ? "OUT" : "IN");
@@ -77,35 +80,15 @@ int fsize, int dir, struct tm *timestamp) {
     }
 }
 
-void icqhistory::putmail(string nick, string email, string msg, int mailt,
-struct tm *timestamp) {
+void icqhistory::putmail(const string nick, const string email,
+const string msg, int mailt, struct tm *timestamp) {
 }
 
-void icqhistory::putcontact(unsigned int uin, icqcontactmsg *conts,
-int dir, struct tm *timestamp) {
-    FILE *f = open(uin, "a");
-    icqcontactmsg *cont;
-    char buf[64];
-
-    if(f) {
-	fprintf(f, "\f\n");
-	fprintf(f, "%s\nCONTACT\n", dir == HIST_MSG_OUT ? "OUT" : "IN");
-	fprintf(f, "%lu\n%lu\n", mktime(timestamp), time(0));
-
-	for(cont = conts; cont; ) {
-	    fprintf(f, "%s\t%lu\n", mime(buf, cont->nick), cont->uin);
-	    cont = (icqcontactmsg *) cont->next;
-	}
-
-	fclose(f);
-    }
-}
-
-bool icqhistory::opencontact(unsigned int uin, time_t lastread = 0) {
+bool icqhistory::opencontact(const imcontact cinfo, time_t lastread) {
     bool ret = true;
     FILE *f;
 
-    if(f = open(uin, "r")) {
+    if(f = open(cinfo, "r")) {
 	opens.push_back(storedopen(f));
     } else {
 	ret = false;
@@ -163,7 +146,7 @@ int icqhistory::setposlastread(time_t lr) {
     return so->rn;
 }
 
-int icqhistory::find(const string sub, int pos = 0) {
+int icqhistory::find(const string sub, int pos) {
     int evt, dir, n, lastfound = -1;
     time_t lr;
     struct tm tsent;
@@ -188,8 +171,7 @@ int icqhistory::find(const string sub, int pos = 0) {
     return lastfound;
 }
 
-int icqhistory::readevent(int &event, time_t &lastread, struct tm &sent,
-int &dir) {
+int icqhistory::readevent(int &event, time_t &lastread, struct tm &sent, int &dir) {
     int ml = 1, thisevent, thisdir;
     char buf[512];
     bool finished, mread = false;
@@ -216,9 +198,7 @@ int &dir) {
 		break;
 	    case 2:
 		if(!strcmp(buf, "URL")) thisevent = EVT_URL; else
-		if(!strcmp(buf, "MSG")) thisevent = EVT_MSG; else
-		if(!strcmp(buf, "FILE")) thisevent = EVT_FILE; else
-		if(!strcmp(buf, "CONTACT")) thisevent = EVT_CONTACT;
+		if(!strcmp(buf, "MSG")) thisevent = EVT_MSG;
 		break;
 	    case 3:
 		memcpy(&tm, localtime(&(t = atol(buf))), sizeof(tm));
@@ -247,7 +227,7 @@ void icqhistory::getmessage(string &text) {
     vector<storedopen>::iterator so = opens.end()-1;
     vector<string>::iterator i;
 
-    for(i = so->lastevent.begin(), text = ""; i != so->lastevent.end(); i++) {
+    for(i = so->lastevent.begin(), text = ""; i != so->lastevent.end(); ++i) {
 	if(text.size()) text += "\n";
 	text += *i;
     }
@@ -259,7 +239,7 @@ void icqhistory::geturl(string &url, string &text) {
 
     url = text = "";
 
-    for(i = so->lastevent.begin(); i != so->lastevent.end(); i++)
+    for(i = so->lastevent.begin(); i != so->lastevent.end(); ++i)
     if(i == so->lastevent.begin()) url = *i; else {
 	if(text.size()) text += "\n";
 	text += *i;
@@ -278,37 +258,6 @@ void icqhistory::getfile(unsigned long &seq, string &fname, int &fsize) {
     }
 }
 
-void icqhistory::getcontact(icqcontactmsg **cont) {
-    int len;
-    const char *p;
-    icqcontactmsg *cc;
-    vector<string>::iterator i;
-    vector<storedopen>::iterator so = opens.end()-1;
-
-    *cont = 0;
-
-    for(i = so->lastevent.begin(); i != so->lastevent.end(); i++)
-    if(p = strrchr(i->c_str(), '\t')) {
-	if(!*cont) {
-	    cc = new icqcontactmsg;
-	    *cont = cc;
-	} else {
-	    cc->next = new icqcontactmsg;
-	    cc = (icqcontactmsg *) cc->next;
-	    cc->next = 0;
-	}
-
-	cc->uin = strtoul(p+1, 0, 0);
-	len = p-i->c_str();
-	if(len > 31) len = 31;
-	strncpy(cc->nick, i->c_str(), len);
-	cc->nick[len] = 0;
-	unmime(cc->nick);
-	strim(cc->nick);
-	cc->next = 0;
-    }
-}
-
 void icqhistory::closecontact() {
     if(!opens.empty()) {
 	vector<storedopen>::iterator so = opens.end()-1;
@@ -317,18 +266,17 @@ void icqhistory::closecontact() {
     }
 }
 
-void icqhistory::fillmenu(unsigned int uin, verticalmenu *m) {
+void icqhistory::fillmenu(const imcontact cinfo, verticalmenu *m) {
     int event, dir, color, i, n = 0;
     struct tm tm;
     string text, url;
-    icqcontactmsg *contact, *curcont;
     time_t lastread = 0;
     char buf[512];
 
     list<histentry> hitems;
     list<histentry>::iterator ii;
     
-    if(opencontact(uin)) {
+    if(opencontact(cinfo)) {
 	while((n = readevent(event, lastread, tm, dir)) != -1) {
 	    url = text = "";
 
@@ -340,22 +288,17 @@ void icqhistory::fillmenu(unsigned int uin, verticalmenu *m) {
 		    geturl(url, text);
 		    text = url + " " + text;
 		    break;
-		case EVT_CONTACT:
-		    getcontact(&contact);
-		    for(curcont = contact; curcont; curcont = (icqcontactmsg *) curcont->next) {
-			if(text.size()) text += ", ";
-			text += (string) curcont->nick + " " + i2str(curcont->uin);
-		    }
-		    break;
 	    }
 
-	    if(!uin) text.replace(0, 1, "");
+	    if(cinfo.empty()) text.replace(0, 1, "");
 
 	    if(text.size()) {
+		if(text.size() > COLS) text.erase(COLS);
 		text = (string) " " + time2str(&lastread, "DD.MM hh:mm", buf) + " " + text;
 		color = dir == HIST_MSG_IN ? conf.getcolor(cp_main_text) : conf.getcolor(cp_main_highlight);
 
 		histentry h;
+
 		h.text = text;
 		h.lastread = n;
 		h.color = color;
