@@ -7,9 +7,9 @@
 
 struct imsearchparams {
     imsearchparams() {
-	onlineonly = false;
+	onlineonly = sincelast = false;
 	uin = 0;
-	minage = maxage = country = language = 0;
+	minage = maxage = country = language = randomgroup = 0;
 	gender = genderUnspec;
 	agerange = ICQ2000::range_NoRange;
     };
@@ -19,12 +19,12 @@ struct imsearchparams {
     };
 
     protocolname pname;
-    bool onlineonly;
+    bool onlineonly, sincelast;
     unsigned int uin;
-    unsigned short minage, maxage, country, language;
+    unsigned short minage, maxage, country, language, randomgroup;
     ICQ2000::AgeRange agerange;
     imgender gender;
-    string firstname, lastname, nick, city, state;
+    string firstname, lastname, nick, city, state, kwords;
     string company, department, position, email, room;
 
     void save(const string &prname) const;
@@ -44,6 +44,9 @@ class imcontroller {
 	void aimupdateprofile();
 	void ircchannels();
 
+	void icqsynclist();
+	void yahoosynclist();
+
     public:
 	imcontroller();
 	~imcontroller();
@@ -51,6 +54,7 @@ class imcontroller {
 	void registration(icqconf::imaccount &account);
 	void updateinfo(icqconf::imaccount &account);
 	void channels(icqconf::imaccount &account);
+	void synclist(icqconf::imaccount &account);
 };
 
 extern imcontroller imcontrol;
